#========================================================================================
# package      API::Patch::PatchHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This the install handler for all patch install functions.
#========================================================================================
package API::Patch::PatchHandler;

use strict;
use Scalar::Util qw (blessed);
use Time::HiRes qw (time);

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler = API::Patch::PatchHandler->new($PatchSystemInfoHandler);
#----------------------------------------------------------------------------------------
# description  Creates a new patch handler object of API::Patch::PatchHandler.
#               Note: abstract class
#----------------------------------------------------------------------------------------
# input        $PatchSystemInfoHandler | patch system info handler API::Patch::PatchSystemInfoHandler | object
# return       $PatchHandler | patch handler object | object
#========================================================================================
sub new {
    my $class = shift;
    my ($PatchSystemInfoHandler) = @_;

    die "missing patch system info handler\n\n" unless (defined $PatchSystemInfoHandler);

    #return object
    return  bless {'PatchSystemInfoHandler' => $PatchSystemInfoHandler}, $class;
}

#========================================================================================
# function     doPatch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch($TargetVersion);
#----------------------------------------------------------------------------------------
# description  check if a versionized doPatch_X_Y_Z() function exists, else it executes
#               doPatch_universal()
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    my $PatchHandler = blessed($self);
    my $InheritFunctionName = $PatchHandler . '::doPatch';

    #get patch system info handler for register informations
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();

    my $DoneWork = $PatchSystemInfoHandler->getProcessedHandlerPrognosis();
    my $PastTimeMessage = $PatchSystemInfoHandler->getPastTimeMessage();

    #print and log working access
    print "\n$DoneWork\% of patch work completed, $PastTimeMessage.\n";
    $self->logPatchHistory( $TargetVersion, "$DoneWork\% of patch work completed, $PastTimeMessage.", 3);

    #check if function can be executet
    unless ($self->toExecute($TargetVersion)){
        $self->logPatchHistory( $TargetVersion, "skip function because it has already version $TargetVersion");
        return;
    }

    #check the precondition
    $self->doPatch_checkPreCondition($TargetVersion);

    #get function name
    my $PatchFunctionName = "doPatch_$TargetVersion";
    $PatchFunctionName =~ s/\./_/g;

    #check if function exists, call doPatch_X_Y_Z if exists and doPatch_universal if not exists
    if ($self->can($PatchFunctionName) ) {#call existing versionized doPatch_X_Y_Z() function
        #RODWORKS think about logging
        $self->$PatchFunctionName;
    }
    else {#call standard doPatch_universal() function
        #ROADWORKS logg missing versionized patch function

        $self->doPatch_universal($TargetVersion);
    }

    #set new handler version and log patch history
    $PatchSystemInfoHandler->setCurrentHandlerVersion($PatchHandler,$TargetVersion);

    #log patch history
    $self->logPatchHistory( $TargetVersion, "set handler version to $TargetVersion", 3);

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     toExecute
# state        public
#----------------------------------------------------------------------------------------
# syntax       my CanExecute = $Handler->toExecute($TargetVersion);
#----------------------------------------------------------------------------------------
# description  check if handler function can be executed
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
# return       $ReadyToExecute | true if handler functions are ready to execute | bool
#========================================================================================
sub toExecute{
    my $self = shift;
    my ($TargetVersion) = @_;

    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();
    my $PatchHandler = blessed($self);

    my $CurrentHandlerVersion = $PatchSystemInfoHandler->getCurrentHandlerVersion($PatchHandler);

    if ($PatchSystemInfoHandler->versionLowerThan($CurrentHandlerVersion, $TargetVersion)){
        #log patch history
        $self->logPatchHistory(
            $TargetVersion,
            "handler ready to patch from version $CurrentHandlerVersion to version $TargetVersion",
        );
        return 1;
    }
    else {
        #log patch history
        $self->logPatchHistory(
            $TargetVersion,
            "handler has already higher version $CurrentHandlerVersion patch to version $TargetVersion not necessary"
        );
        return 0;
    }
}

#========================================================================================
# function     doPatch_checkPreCondition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_checkPreCondition($TargetVersion);
#----------------------------------------------------------------------------------------
# description  check pre condition for each handler using information from patch system info handler
#               note: abstract function
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_checkPreCondition{
    my $self = shift;
    my ($TargetVersion) = @_;

    die 'implement abstract method doPatch_checkPreCondition()';
}

#========================================================================================
# function     doPatch_universal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_universal($TargetVersion);
#----------------------------------------------------------------------------------------
# description  standard patch function for handler
#               note: abstract function
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_universal{
    my $self = shift;
    my ($TargetVersion) = @_;

    die 'implement abstract method patch_universal()';
}

#========================================================================================
# function     doPatch_basic
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_basic($TargetVersion);
#----------------------------------------------------------------------------------------
# description  basic patch function for handler, normally call sub patch handler objects
#               note: abstract function
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_basic{
    my $self = shift;
    my ($TargetVersion) = @_;

    die 'implement abstract method patch_basic()';
}

#========================================================================================
# function     registerStart
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->registerStart($TargetVersion);
#----------------------------------------------------------------------------------------
# description  register function in history of patch system info handler
#----------------------------------------------------------------------------------------
# input        $FunctionName | name of call function | string
# input        $Version | patch target version | string
#========================================================================================
sub registerStart{
    my $self = shift;
    my ($Version) = @_;

    my $PatchHandler = blessed($self);
    my $FunctionName = (caller(1))[3];

    #get only function name
    $FunctionName =~ s/.*(::[\w\_]+)$/$1/;

    my $InheritFunctionName = $PatchHandler . $FunctionName;

    #get patch system info handler for register informations
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();

    #register start
    $PatchSystemInfoHandler->registerStartedPatchFunction($Version, $InheritFunctionName);

    #log patch history
    $PatchSystemInfoHandler->logPatchHistory(
        $Version,
        $InheritFunctionName,
        'start patch function',
    );

    return;
}

#========================================================================================
# function     registerEnd
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->registerEnd($FunctionName, $Version);
#----------------------------------------------------------------------------------------
# description  register function in history of patch system info handler
#----------------------------------------------------------------------------------------
# input        $FunctionName | name of call function | string
# input        $Version | patch target version | string
#========================================================================================
sub registerEnd{
    my $self = shift;
    my ($Version) = @_;

    my $PatchHandler = blessed($self);
    my $FunctionName = (caller(1))[3];

    #get only function name
    $FunctionName =~ s/.*(::[\w\_]+)$/$1/;

    my $InheritFunctionName = $PatchHandler . $FunctionName;

    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();

    #register end
    $PatchSystemInfoHandler->registerCompletedPatchFunction($Version, $InheritFunctionName);

    #log patch history
    $PatchSystemInfoHandler->logPatchHistory(
        $Version,
        $InheritFunctionName,
        'finish patch function',
    );

    return;
}

#========================================================================================
# function     getPatchSystemInfoHandler
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $PatchSystemInfoHandler = $Handler->getPatchSystemInfoHandler();
#----------------------------------------------------------------------------------------
# description  check if handler function can be executed
#----------------------------------------------------------------------------------------
# return       $PatchSystemInfoHandler | patch system info handler object | object
#========================================================================================
sub getPatchSystemInfoHandler{
    my $self = shift;

    return $self->{'PatchSystemInfoHandler'};
}

#========================================================================================
# function     callSubHandlerWithVersionSources
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->callSubHandlerWithVersionSources($TargetVersion, $SubHandlerName);
#----------------------------------------------------------------------------------------
# description  this function generates a new perl process. the process uses different perl sources
#               find in the specific version folder. the call-PatchHandler.pl script is called
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
# input        $Handler | name of the patch handler | string
#========================================================================================
sub callSubHandlerWithVersionSources{
    my $self = shift;
    my ($TargetVersion, $Handler) = @_;

    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();
    my $VersionFolder = $PatchSystemInfoHandler->getCartridgesVersionFolder($TargetVersion);

    #log patch history
    $self->logPatchHistory($TargetVersion, "call handler '$Handler', use source '$VersionFolder'");

    #require DE_EPAGES::Core::API::SystemCommand qw (SystemCommand);
    $self->useNow('DE_EPAGES::Core::API::SystemCommand', qw (SystemCommand));

    {#version call
        #set environment variables for system command
        local $ENV{EPAGES_CARTRIDGES} = $VersionFolder;

        #check for OS differences
        my $PerlExe;
        my $OperationgSystem = $PatchSystemInfoHandler->getSystemInfo('OperatingSystem');
        if ($OperationgSystem eq 'MSWin32') {
            $PerlExe = $^X;
        } else {
            $PerlExe = ". $ENV{PERL}; EPAGES_CARTRIDGES=$ENV{EPAGES_CARTRIDGES} $^X";
        }

        #get infos about to call script
        my $PatchCallHandlerScript = $PatchSystemInfoHandler->getSystemInfo('PatchCallHandlerScript');
        my $PatchSystemInfoFile = $PatchSystemInfoHandler->getSystemInfo('PatchSystemInfoFile');

        # perl(win/lin) call patch handler
        SystemCommand(
            "$PerlExe $PatchCallHandlerScript -handler $Handler -patchinfofile $PatchSystemInfoFile",
            {'Print' => 1}
        );
    }

    return;
}

sub logPatchHistory{
    my $self = shift;
    my ($TargetVersion,$LogLine,$LogLevel) = @_;

    my $PatchHandler = blessed($self);
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();
    my $FunctionName = (caller(1))[3];

    #get only function name
    $FunctionName =~ s/.*(::[\w\_]+)$/$1/;

    #log patch history
    $PatchSystemInfoHandler->logPatchHistory(
        $TargetVersion,
        $PatchHandler . $FunctionName,
        $LogLine,
        $LogLevel
    );

    return;
}

sub useNow{
    my $self=shift;
    my ($Package, @Functions, $Dir) = @_;

    #get handler and target version for logging
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();
    my $PatchHandler = blessed($self);
    my $CurrentVersion = $PatchSystemInfoHandler->getCurrentHandlerVersion($PatchHandler);
    my $TargetVersion = $PatchSystemInfoHandler->nextVersion($CurrentVersion);

    #log patch history
    $self->logPatchHistory($TargetVersion, "require $Package");

    #get filename for package
    my $FileName = "$Package.pm";

    #replace '::' <-> '/' in file name
    $FileName =~ s/::/\//g;

    #require file
    require $FileName;
    die $@ if $@;

    #compile functions
    {
        #set package to import, package that call this function
        local $Exporter::ExportLevel = 1;

        #import function to calling package
        $Package->import( @Functions );
    }

    return;
}

1;