#========================================================================================
# package      API::Patch::PatchPrepareHandlerVersionFolders
# state        public
#----------------------------------------------------------------------------------------
# description  This the install handler for all patch install functions.
#========================================================================================
package API::Patch::PatchPrepareHandlerVersionFolders;
use base API::Patch::PatchHandler;

use strict;

#========================================================================================
# function     doPatch_checkPreCondition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_checkPreCondition($TargetVersion);
#----------------------------------------------------------------------------------------
# description  check pre condition for each handler using information from patch system info handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_checkPreCondition{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #use now DE_EPAGES::Core::API::File qw (ExistsDirectory);
    $self->useNow('DE_EPAGES::Core::API::File', qw ( DeleteDirectory ExistsDirectory ));

    #get patch system info handler
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();

    #get info
    my $StartVersion = $PatchSystemInfoHandler->getSystemInfo('ePagesStartVersion');
    my $CurrentVersion = $PatchSystemInfoHandler->previousVersion($TargetVersion);
    my $CartridgesDir = $PatchSystemInfoHandler->getSystemInfo('EpagesCartridgesDir');
    my $CartridgesVersionFolder = $PatchSystemInfoHandler->getCartridgesVersionFolder($TargetVersion);

    #check that current version i higher than the start version
    die "current version '$CurrentVersion' is smaller than the start version '$StartVersion'"
        if ($PatchSystemInfoHandler->versionLowerThan($CurrentVersion,$StartVersion));

    #check if current version not equal to start version
    if ($StartVersion eq $CurrentVersion){

        #check if cartridges dir exists
        die "cartridge directory '$CartridgesDir' does not exists" unless (ExistsDirectory($CartridgesDir));

        #check if cartridges version directory exists
        if (ExistsDirectory($CartridgesVersionFolder)){

            #log patch history
            $self->logPatchHistory( $TargetVersion, "delete directory '$CartridgesVersionFolder'", 2 );

            #delete directory if exists its in the way
            DeleteDirectory($CartridgesVersionFolder, {'recursive' => 1});
        }
    }
    else{
        #check if previous version folder exists
        my $CurrentVersionFolder = $PatchSystemInfoHandler->getCartridgesVersionFolder($CurrentVersion);
        my $TargetVersionFolder = $PatchSystemInfoHandler->getCartridgesVersionFolder($TargetVersion);

        die "cant generate version folder '$TargetVersionFolder' because previous version folder '$CurrentVersionFolder' does not exists"
            unless(ExistsDirectory($CurrentVersionFolder));
    }

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_universal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_universal($TargetVersion);
#----------------------------------------------------------------------------------------
# description  standard patch function for handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_universal{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #register step
    $self->logPatchHistory(
        $TargetVersion,
        'summary: generate epages cartridges version folders',
        4
    );

    $self->doPatch_basic($TargetVersion);

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_basic
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_basic($TargetVersion);
#----------------------------------------------------------------------------------------
# description  basic patch function for handler, normally call sub patch handler objects
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_basic{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #use now
    $self->useNow('DE_EPAGES::Core::API::File',
        qw (
            GetInfoDirectory
            ExistsDirectory
            CopyFile
            CreateDirectory
            ExistsFile
            CopyDirectory
        )
    );

    #get patch system info handler
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();

    #get info
    my $StartVersion = $PatchSystemInfoHandler->getSystemInfo('ePagesStartVersion');
    my $CurrentVersion = $PatchSystemInfoHandler->previousVersion($TargetVersion);
    my $NewVersionFolder = $PatchSystemInfoHandler->getCartridgesVersionFolder($TargetVersion);
    my $CurrentSourceFolder = $PatchSystemInfoHandler->getCartridgesVersionFolder($CurrentVersion);
    my $CartridgesDir = $PatchSystemInfoHandler->getSystemInfo('EpagesCartridgesDir');
    my $NewPatchVersionDir = $PatchSystemInfoHandler->getPatchVersionFolder($TargetVersion);

    #set current source folder to epages cartridges dir if current version is the same as start version
    $CurrentSourceFolder = $CartridgesDir if ($CurrentVersion eq $StartVersion);

    #generate log line
    my $LogLine = "generate linked version folder '$NewVersionFolder' from source '$CurrentSourceFolder'";

    #log patch history
    $self->logPatchHistory( $TargetVersion, $LogLine, 2 );

    #get all source directory/file information
    my $hInfo = GetInfoDirectory($CurrentSourceFolder, {'recursive' => 1});

    #get files and directories
    my @Files = @{$hInfo->{'Files'}};
    my @SubDirs = @{$hInfo->{'Directories'}};

    #create linked directories
    foreach my $SubDir (sort @SubDirs){

        #clean sub dir
        $SubDir =~ s/$CurrentSourceFolder\///;

        #create directory
        CreateDirectory("$NewVersionFolder/$SubDir");
    }

    #create linked files
    foreach my $File (@Files){
        $File =~ s/$CurrentSourceFolder\///;

        unless (ExistsFile("$NewVersionFolder/$File")){

            #copy or generate hard linked file if not exists
            CopyFile(
                "$CurrentSourceFolder/$File",
                "$NewVersionFolder/$File",
                {link => 'orCopy', stat => 'ignore'}
            );
        }
    }

    #log patch history
    $self->logPatchHistory(
        $TargetVersion,
        "copy new files from patch folder '$NewPatchVersionDir' to cartridges folder '$NewVersionFolder'",
        2,
    );

    #copy new files from patch folder to cartridges version folder
    CopyDirectory(
        $NewPatchVersionDir,
        $NewVersionFolder,
        undef,
        {link => 'orCopy', unlink => 1, stat => 'ignore'}
    );

    #call sub handler
    $self->callSubHandlerWithVersionSources($TargetVersion, 'PatchPrepareHandlerVersionFoldersPatchCartridgeDir');

    #register end
    $self->registerEnd($TargetVersion);
}

#example versionized function
##========================================================================================
## function     doPatch_6_14_2
## state        public
##----------------------------------------------------------------------------------------
## syntax       $Handler->doPatch_universal();
##----------------------------------------------------------------------------------------
## description  standard patch function for handler
##========================================================================================
#sub doPatch_6_14_2{
#    my $self = shift;
#
#    my $TargetVersion = '6.14.2';
#
#    #register start
#    $self->registerStart($TargetVersion);
#
#    print "\n\nROADWORKS - SpadeWork for PatchInstallHandler::doPatch_6_14_2";
#
#    $self->doPatch_basic($TargetVersion);
#
#    #register end
#    $self->registerEnd($TargetVersion);
#
#    print "\n\nROADWORKS - ReWork for PatchInstallHandler::doPatch_6_14_2";
#
#    return;
#}

#----------------------------------------------------Patch Functions--------------------------

1;