#========================================================================================
# package      API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir
# state        public
#----------------------------------------------------------------------------------------
# description  This the install handler for all patch install functions.
#========================================================================================
package API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir;
use base API::Patch::PatchHandler;

use strict;

#========================================================================================
# function     doPatch_checkPreCondition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_checkPreCondition($TargetVersion);
#----------------------------------------------------------------------------------------
# description  check pre condition for each handler using information from patch system info handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_checkPreCondition{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #use now DE_EPAGES::Core::API::File qw (ExistsDirectory);
    $self->useNow('DE_EPAGES::Core::API::File', qw (ExistsDirectory) );

    #get patch system info handler
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();

    #get info
    my $CartridgesVersionDir = $PatchSystemInfoHandler->getCartridgesVersionFolder($TargetVersion);

    #check conditions
    unless (ExistsDirectory($CartridgesVersionDir)){
        my $LogLine = "missing directory '$CartridgesVersionDir'";

        #log patch history
        $self->logPatchHistory( $TargetVersion, $LogLine, 2 );

        die "\n" . $LogLine . "\n";
    }

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_universal
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_universal($TargetVersion);
#----------------------------------------------------------------------------------------
# description  standard patch function for handler
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_universal{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #register step
    $self->logPatchHistory(
        $TargetVersion,
        'summary: delete all redundant files in version folder',
        4
    );

    $self->doPatch_basic($TargetVersion);

    #register end
    $self->registerEnd($TargetVersion);

    return;
}

#========================================================================================
# function     doPatch_basic
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->doPatch_basic($TargetVersion);
#----------------------------------------------------------------------------------------
# description  basic patch function for handler, normally call sub patch handler objects
#----------------------------------------------------------------------------------------
# input        $TargetVersion | target version to patch the patch handler | string
#========================================================================================
sub doPatch_basic{
    my $self = shift;
    my ($TargetVersion) = @_;

    #register start
    $self->registerStart($TargetVersion);

    #use now
    $self->useNow('DE_EPAGES::Core::API::File', qw ( GetSubDirectories ExistsFile ));
    $self->useNow('DE_EPAGES::Core::API::PerlTools', qw( UsePackage ));

    #get patch system info handler
    my $PatchSystemInfoHandler = $self->getPatchSystemInfoHandler();

    #get info
    my $CartridgesDir = $PatchSystemInfoHandler->getSystemInfo('EpagesCartridgesDir');
    my $PreviousVersion = $PatchSystemInfoHandler->previousVersion($TargetVersion);

    #generate log line
    my $LogLine = "get all cartridges installer packages to call patchCartridgeDir('$PreviousVersion')";

    #log patch history
    $self->logPatchHistory( $TargetVersion, $LogLine, 1 );

    #get cartridge installer packages
    my @Packages;
    my $aVendorDirs = GetSubDirectories($CartridgesDir, {'recursive' => 0});
    foreach my $VendorDir ( @$aVendorDirs ) {
        my $aCartridgeDirs = GetSubDirectories($VendorDir, {'recursive' => 0});
        foreach my $CartridgeDir (@$aCartridgeDirs) {
            if ( ExistsFile($CartridgeDir . '/API/Cartridge.pm') ) {
                my $Package = $CartridgeDir;
                $Package =~ s/$CartridgesDir//;
                $Package =~ s/^\///;
                $Package =~ s/\//::/g;
                push @Packages, $Package;
            }
        }
    }

    #for all cartridges
    foreach my $Package (@Packages ) {

        #get package object
        my $CartridgePackage = $Package . '::API::Cartridge';

        #log patch history
        $self->logPatchHistory( $TargetVersion, "call $CartridgePackage->patchCartridgeDir('$PreviousVersion')", 0 );

        #use package
        UsePackage($CartridgePackage);

        #construct object
        my $Cartridge = $CartridgePackage->new();

        #call patch cartridge dir, delete redundant files
        $Cartridge->patchCartridgeDir($PreviousVersion) if $Cartridge->can('patchCartridgeDir');

    }

    #register end
    $self->registerEnd($TargetVersion);
}

#example versionized function
##========================================================================================
## function     doPatch_6_14_2
## state        public
##----------------------------------------------------------------------------------------
## syntax       $Handler->doPatch_universal();
##----------------------------------------------------------------------------------------
## description  standard patch function for handler
##========================================================================================
#sub doPatch_6_14_2{
#    my $self = shift;
#
#    my $TargetVersion = '6.14.2';
#
#    #register start
#    $self->registerStart($TargetVersion);
#
#    print "\n\nROADWORKS - SpadeWork for PatchInstallHandler::doPatch_6_14_2";
#
#    $self->doPatch_basic($TargetVersion);
#
#    #register end
#    $self->registerEnd($TargetVersion);
#
#    print "\n\nROADWORKS - ReWork for PatchInstallHandler::doPatch_6_14_2";
#
#    return;
#}

#----------------------------------------------------Patch Functions--------------------------

1;