#========================================================================================
# package      API::Patch::PatchSystemInfoHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Roadworks
#========================================================================================
package API::Patch::PatchSystemInfoHandler;

use strict;
use Config;
use Data::Dumper;
use Sys::Hostname;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::File qw (WriteFile ExistsFile GetFileContent AbsolutePath GetFileMD5Digest CompareMD5File);
use DE_EPAGES::Core::API::PerlTools qw (IndexOf );
use DE_EPAGES::Core::API::Cartridge;

#ROADWORKS - checkintegrity function
#ROADWORKS - perl critic
#ROADWORKS - comments
#ROADWORKS - testcases
#ROADWORKS - documentation

sub new {
    my $class = shift;
    my ($PatchSystemInfoFile) = @_;

    #check for file name
    die 'missing patch system info file' unless (defined $PatchSystemInfoFile);

    $PatchSystemInfoFile = AbsolutePath($PatchSystemInfoFile);
    $PatchSystemInfoFile =~ s/\\/\//g;

    my $PatchDir = $PatchSystemInfoFile;

    #check if
    die "file path not valid, $PatchSystemInfoFile" unless ($PatchDir =~ /\/Data\//);
    $PatchDir =~ s/\/Data\/.*//;


    #return object
    my $self =  bless {
        #patch system info file
        'PatchSystemInfoFile' => $PatchSystemInfoFile,

        #ground functions with log level
        'ValidGroundFunctions' => {
            'doPatch' => 4,
            'callSubHandlerWithVersionSources' => 3,
            'toExecute' => 2,
            'doPatch_checkPreCondition' => 2,
            'doPatch_basic' => 1,
            'doPatch_universal' => 0,
            'useNow' => 0,
        },
    }, $class;

    if(ExistsFile($PatchSystemInfoFile)){
        #get object from file
        $self->_collectSystemInfoFromFile();
    }
    else{
        $self->{'PatchSystemInfo'} = {
            'CompletedPatchFunctions'    => {},
            'StartedPatchFunctions'      => {},
            'VersionPath'            => [],
            'ePagesTargetVersion'   => '',
        };
        $self->{'PatchSystemInfoFileHaveToBuild'} = 1;
    }

    #save patchdir
    $self->setSystemInfo('PatchDir',$PatchDir);

    #save start time unless exists
    $self->setStartTime();

    $self->_saveToFile();

    return $self;
}

sub validKey{
    my $self = shift;
    my ($Key) = @_;

    my $hValidKeys = {
        'EpagesSharedDir' => 1,
        'HostName' => 1,
        'PatchDir' => 1,
        'EpagesCartridgesDir' => 1,
        'EpagesCartridgesVersionDir' => 1,
        'EpagesConfigDir' => 1,
        'OperatingSystem' => 1,
        'DataBaseSystem' => 1,
        'OperatingSystemVersion' => 1,
        'DataBaseSystemVersion' => 1,
        'ePagesStartVersion' => 1,
        'ePagesTargetVersion' => 1,
        'ePagesCurrentVersion' => 1,
        'PerlStartVersion' => 1,
        'PerlTargetVersion' => 1,
        'JavaStartVersion' => 1,
        'JavaTargetVersion' => 1,
        'PatchDataDir' => 1,
        'PatchTargetDir' => 1,
        'PatchBackupDir' => 1,
        'PatchLogDir' => 1,
        'PatchCallHandlerScript' => 1,
        'PatchSystemInfoFile' => 1,
        'PatchVersionDir' => 1,
    };

    if (exists $hValidKeys->{$Key}){
        return 1;
    }
    else{
        return 0;
    }
}

sub getPatchTag{
    my $self = shift;

    #get object from file
    $self->_collectSystemInfoFromFile();

    my $EpagesTargetVersion = $self->getSystemInfo('ePagesTargetVersion');
    $EpagesTargetVersion =~ s/\./\_/g;

    return "patch_$EpagesTargetVersion";
}

#ROADWORKS divide in collect static system info and collect variable system info
#ROADWORKS collect static system info should be called at the beginning of the patch
#ROADWORKS collect variable info should be called bofore every patch handler
sub collectSystemInfo{
    my $self = shift;
    my ($hOptions) = @_;

    my $OS = $Config{'osname'};
    my $DBS = 'ROADWORKS';
    my $OSVersion = 'ROADWORKS';
    my $DBSVersion = 'ROADWORKS';
    my $EpagesStartVersion = DE_EPAGES::Core::API::Cartridge->new()->version();
    my $PerlStartVersion = 'ROADWORKS';
    my $PerlTargetVersion ='ROADWORKS';
    my $JavaStartVersion ='ROADWORKS';
    my $JavaTargetVersion ='ROADWORKS';
    my @Versions = @{$self->validVersions()};
    my $EpagesCartridgesDir = $ENV{EPAGES_CARTRIDGES};
    $EpagesCartridgesDir =~ s/\\/\//g;
    my $EpagesCartridgesVersionDir = "$EpagesCartridgesDir/.version";
    my $HostName = hostname();
    my $EpagesSharedDir = $ENV{EPAGES_SHARED};
    $EpagesSharedDir =~ s/\\/\//g;
    my $EpagesConfigDir = $ENV{'EPAGES_CONFIG'};
    $EpagesConfigDir =~ s/\\/\//g;
    my $PatchTag = $self->getPatchTag();
    my $PatchDataDir = $EpagesSharedDir . "/Patches/var-$PatchTag";
    my $PatchDir = $self->getSystemInfo('PatchDir');
    my $PatchVersionDir = $PatchDir . '/Cartridges.v';

    #static system info
    $self->setSystemInfo('EpagesSharedDir', $EpagesSharedDir) unless $self->existsSystemInfo('EpagesSharedDir');
    $self->setSystemInfo('EpagesConfigDir', $EpagesSharedDir) unless $self->existsSystemInfo('EpagesSharedDir');
    $self->setSystemInfo('EpagesCartridgesDir', $EpagesCartridgesDir) unless $self->existsSystemInfo('EpagesCartridgesDir');
    $self->setSystemInfo('EpagesCartridgesVersionDir', $EpagesCartridgesVersionDir) unless $self->existsSystemInfo('EpagesCartridgesVersionDir');
    $self->setSystemInfo('HostName', $HostName) unless $self->existsSystemInfo('HostName'); #HostName
    $self->setSystemInfo('OperatingSystem', $OS) unless $self->existsSystemInfo('OperatingSystem'); #operationg system
    $self->setSystemInfo('DataBaseSystem', $DBS) unless $self->existsSystemInfo('DataBaseSystem'); #database system
    $self->setSystemInfo('PerlTargetVersion', $PerlTargetVersion) unless $self->existsSystemInfo('PerlTargetVersion'); #perl target version
    $self->setSystemInfo('JavaTargetVersion', $JavaTargetVersion) unless $self->existsSystemInfo('JavaTargetVersion'); #java target version
    $self->setSystemInfo('JavaStartVersion', $JavaStartVersion) unless $self->existsSystemInfo('JavaStartVersion'); #java start version

    #variable system info
    $self->setSystemInfo('OperatingSystemVersion', $OSVersion); #operationg system version
    $self->setSystemInfo('DataBaseSystemVersion', $DBSVersion); #database system version

    #static patch info
    $self->setSystemInfo('PatchDataDir', $PatchDataDir) unless $self->existsSystemInfo('PatchDataDir'); #ROADWORKS comments
    $self->setSystemInfo('PatchTargetDir', $PatchDataDir . '/tgt/' . $HostName ) unless $self->existsSystemInfo('PatchTargetDir'); #ROADWORKS comments
    $self->setSystemInfo('PatchBackupDir', $PatchDataDir . '/backup/' . $HostName ) unless $self->existsSystemInfo('PatchBackupDir'); #ROADWORKS comments
    $self->setSystemInfo('PatchLogDir', $PatchDataDir . '/logs/' . $HostName ) unless $self->existsSystemInfo('PatchLogDir'); #ROADWORKS comments
    $self->setSystemInfo('PatchCallHandlerScript', $PatchDir . '/Scripts/call-PatchHandler.pl') unless $self->existsSystemInfo('PatchCallHandlerScript');
    $self->setSystemInfo('PatchVersionDir', $PatchVersionDir) unless $self->existsSystemInfo('PatchVersionDir');

    #static epages info
    $self->setSystemInfo('ePagesStartVersion', $EpagesStartVersion) unless $self->existsSystemInfo('ePagesStartVersion');
    $self->setSystemInfo('ePagesTargetVersion', $Versions[-1]) unless $self->existsSystemInfo('ePagesTargetVersion'); #get last element of versions

    #variable epages info
    $self->setSystemInfo('ePagesCurrentVersion', $EpagesStartVersion); #ePages version before patch e.g 6.13.0


    #set handler Informations
    foreach my $Handler (@{$self->validHandlers()}){
        $self->setCurrentHandlerVersion($Handler, $EpagesStartVersion) if (
            (not $self->existsCurrentHandlerVersion($Handler)) or
            $hOptions->{'overwrite'} == 1
        );
    }

    #static perl info
    $self->setSystemInfo('PerlStartVersion', $PerlStartVersion) unless $self->existsSystemInfo('PerlStartVersion'); #ePages version before patch e.g 5.12.3
    $self->setSystemInfo('PerlTargetVersion', 'ROADWORKS') unless $self->existsSystemInfo('PerlTargetVersion'); #ePages version to patch to e.g 5.13.0

    #static java info
    $self->setSystemInfo('JavaStartVersion', 'ROADWORKS') unless $self->existsSystemInfo('JavaStartVersion'); #java version before patch e.g 1.6
    $self->setSystemInfo('JavaTargetVersion', 'ROADWORKS') unless $self->existsSystemInfo('JavaTargetVersion'); #java version to patch to e.g 1.8

    #ROADWORKS
    #PatchHistory

    $self->_saveToFile();
}

sub _collectSystemInfoFromFile{
    my $self = shift;

    my $hPatchSystemInfo1;

    #remember md5
    $self->{'PatchSystemInfoFileMD5'} = GetFileMD5Digest($self->getSystemInfo('PatchSystemInfoFile'));

    #get file content
    my $rContent = GetFileContent($self->getSystemInfo('PatchSystemInfoFile'));

    eval $$rContent; ##no critic qw(BuiltinFunctions::ProhibitStringyEval)

    $self->{'PatchSystemInfo'} = $hPatchSystemInfo1;

    return 1;
}

#========================================================================================
# function     _saveToFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       _saveToFile(ROADWORKS);
#----------------------------------------------------------------------------------------
# example      ROADWORKS
#----------------------------------------------------------------------------------------
# description  ROADWORKS
#========================================================================================
sub _saveToFile{
    my $self = shift;

    {
        #set local variable for dumper output
        local $Data::Dumper::Varname    = 'hPatchSystemInfo';

        #get file name
        my $OutputFile = $self->getSystemInfo('PatchSystemInfoFile');

        #die if file ore md5 exists without the opposite
        die 'patch system info file exists but no MD5 is set' if(
            (ExistsFile($OutputFile)) and (not exists $self->{'PatchSystemInfoFileMD5'})
        );
        die 'MD5 is defined but patch system info file not exists' if(
            (not ExistsFile($OutputFile)) and (exists $self->{'PatchSystemInfoFileMD5'})
        );

        #check if patch system info file have to build
        unless ($self->{'PatchSystemInfoFileHaveToBuild'}){
            #get MD5 from file
            my $MD5 = $self->{'PatchSystemInfoFileMD5'};

            #check if patch system info file is changed by an other process
            die 'cant save patch system info file, file is changed by an other process' unless(
                CompareMD5File($OutputFile,$MD5)
            );
        }

        #create content for output
        my $Content = Dumper($self->{'PatchSystemInfo'});

        #overwrite file
        WriteFile( $OutputFile, \$Content );

        #get new md5sum from current file
        $self->{'PatchSystemInfoFileMD5'} = GetFileMD5Digest($OutputFile);

        #remember that patch system info file is already build
        $self->{'PatchSystemInfoFileHaveToBuild'} = 0;
    }

    return;
}

sub setSystemInfo{
    my $self = shift;
    my ($Key, $Value) = @_;

    die "unable to set system info for key, $Key is not valid" unless ($self->validKey($Key));
    die "cant set patch system info file" if ($Key eq 'PatchSystemInfoFile');

    $self->{'PatchSystemInfo'}->{$Key} = $Value;

    $self->_saveToFile();
    return;
}

sub existsSystemInfo{
    my $self = shift;
    my ($Key) = @_;

    die "unable to check system info for key, $Key is not valid" unless ($self->validKey($Key));

    return exists $self->{'PatchSystemInfo'}->{$Key};
}

sub validHandlers{
    my $self = shift;

    my @ValidHandlers = (
        #prepare handlers
        'API::Patch::PatchPrepareHandler',
        'API::Patch::PatchPrepareHandlerVersionFolders',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir',
        'API::Patch::PatchPrepareHandlerStoreTypes',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter',
        'API::Patch::PatchPrepareHandlerDojo',
        'API::Patch::PatchPrepareHandlerJavaScript',
        'API::Patch::PatchPrepareHandlerStyleSheet',

        #preinstall handlers
        'API::Patch::PatchPreInstallHandlerWebServer',
        'API::Patch::PatchPreInstallHandlerBackup',
        'API::Patch::PatchPreInstallHandlerFiles',

        #install handlers
        'API::Patch::PatchInstallHandler',
        'API::Patch::PatchInstallHandlerServices',
        'API::Patch::PatchInstallHandlerStores',
        'API::Patch::PatchInstallHandlerFiles',
        'API::Patch::PatchInstallHandlerConfigs',
        'API::Patch::PatchInstallHandlerCartridges',

        #postinstall handlers
        'API::Patch::PatchPostInstallHandler',
        'API::Patch::PatchPostInstallHandlerWebServer',
        'API::Patch::PatchPostInstallHandlerBackup',
        'API::Patch::PatchPostInstallHandlerFiles',
    );

    return \@ValidHandlers;
}

sub validHandler{
    my $self = shift;
    my ($Handler) = @_;

    foreach my $ValidHandler (@{$self->validHandlers()}){
        return 1 if ($Handler eq $ValidHandler);
    }

    return 0;
}

sub validVersions{
    my $self = shift;

    #get object from file
    $self->_collectSystemInfoFromFile();

    return $self->{'PatchSystemInfo'}->{'VersionPath'};
}

sub validVersion{
    my $self = shift;
    my ($Version) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    foreach my $ValidVersion (@{$self->validVersions()}){
        return 1 if ($Version eq $ValidVersion);
    }

    return 0;
}

sub validFunction{
    my $self = shift;
    my ($Function) = @_;

    foreach my $ValidFunction (@{$self->validFunctions()}){
        return 1 if ($Function eq $ValidFunction);
    }

    return 0;
}

sub validGroundFunctions{
    my $self = shift;

    my @ValidGroundFunctions = keys %{$self->{'ValidGroundFunctions'}};

    return \@ValidGroundFunctions;
}

sub getGroundFunctionLogLevel{
    my $self = shift;
    my ($GroundFunctionName) = @_;

    die "$GroundFunctionName is not a valid ground function" unless (exists $self->{'ValidGroundFunctions'}->{$GroundFunctionName});

    return $self->{'ValidGroundFunctions'}->{$GroundFunctionName};
}

sub validFunctions{
    my $self = shift;

    my @ValidFunctions;

    #build result array
    foreach my $ValidHandler (@{$self->validHandlers()}){
        #register all valid functions
        foreach my $ValidGroundFunction (@{$self->validGroundFunctions()}){
            push(@ValidFunctions, $ValidHandler . '::' . $ValidGroundFunction);
        }
        #register versionized doPatch_X_Y_Z functions
        foreach my $ValidVersion (@{$self->validVersions()}){
            my $ValidVersion_underscored = $ValidVersion;
            $ValidVersion_underscored =~ s/\./\_/g;
            push(@ValidFunctions, $ValidHandler . '::doPatch_' . $ValidVersion_underscored);
        }
    }

    return \@ValidFunctions;
}

sub isLastVersion{
    my $self = shift;
    my ($Version) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "$Version is not a valid version" unless ($self->validVersion($Version));

    if ($Version eq $self->validVersions()->[-1]){
        return 1;
    }
    else {
        return 0;
    }
}

sub isFirstVersion{
    my $self = shift;
    my ($Version) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "$Version is not a valid version" unless ($self->validVersion($Version));

    if ($Version eq $self->validVersions()->[0]){
        return 1;
    }
    else {
        return 0;
    }
}

sub lastVersion{
    my $self = shift;

    #get object from file
    $self->_collectSystemInfoFromFile();

    return $self->{'PatchSystemInfo'}->{'VersionPath'}->[-1];
}

sub firstVersion{
    my $self = shift;

    #get object from file
    $self->_collectSystemInfoFromFile();

    return $self->{'PatchSystemInfo'}->{'VersionPath'}->[0];
}

sub nextVersion{
    my $self = shift;
    my ($Version) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "$Version is not a valid version" unless ($self->validVersion($Version));
    die "$Version is the last valid version, no following version exist" if ($self->isLastVersion($Version));

    #get current position
    my $Position = IndexOf($Version, @{$self->validVersions()});

    return $self->validVersions()->[$Position + 1];
}

sub previousVersion{
    my $self = shift;
    my ($Version) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "$Version is not a valid version" unless ($self->validVersion($Version));
    die "$Version is the last valid version, no following version exist" if ($self->isFirstVersion($Version));

    #get current position
    my $CurrentPosition = IndexOf($Version, @{$self->validVersions()});

    return $self->validVersions()->[$CurrentPosition - 1];
}

sub getSystemInfo{
    my $self = shift;
    my ($Key) = @_;

    return $self->{'PatchSystemInfoFile'} if ($Key eq 'PatchSystemInfoFile');

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "unable to get system info for key, $Key is not valid" unless ($self->validKey($Key));
    die "PatchSystemInfo ERROR no entry for key $Key" unless (exists $self->{'PatchSystemInfo'}->{$Key});

    return $self->{'PatchSystemInfo'}->{$Key};
}

sub existsEnvironmentVariable{
    my $self = shift;
    my ($Key) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    #ROADWORKS
}

sub getEnvironmentVariables{
    #ROADWORKS
}

sub registerCompletedPatchFunction{
    my $self = shift;
    my ($Version, $Function) = @_;

    die "unable to register function, $Function is not valid" unless ($self->validFunction($Function));
    die "unable to register version, $Version is not valid" unless ($self->validVersion($Version));
    die "unable to register function, $Function is not valid, missing 'doPatch' in name" unless ($Function =~ /::doPatch/);

    $self->{'PatchSystemInfo'}->{'CompletedPatchFunctions'}->{$Version}->{$Function} = time;

    $self->_saveToFile();
    return;
}

sub isPatchFunctionCompleted{
    my $self = shift;
    my ($Version, $Function) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "unable to register function, $Function is not valid" unless ($self->validFunction($Function));
    die "unable to register version, $Version is not valid" unless ($self->validVersion($Version));

    #check if patch task has an entry
    return 0 if (not exists $self->{'PatchSystemInfo'}->{'CompletedPatchFunctions'}->{$Version}->{$Function});

    #get time
    my $TimeStamp = $self->{'PatchSystemInfo'}->{'CompletedPatchFunctions'}->{$Version}->{$Function};

    return 1 if ($TimeStamp ne ''); #ROADWORKS check if it is a time date
    return 0;
}

sub registerStartedPatchFunction{
    my $self = shift;
    my ($Version, $Function) = @_;

    die "unable to register function, $Function is not valid" unless ($self->validFunction($Function));
    die "unable to register version, $Version is not valid" unless ($self->validVersion($Version));
    die "unable to register function, $Function is not valid, missing 'doPatch' in name" unless ($Function =~ /::doPatch/);

    $self->{'PatchSystemInfo'}->{'StartedPatchFunctions'}->{$Version}->{$Function} = time;

    $self->_saveToFile();
    return;
}

sub isPatchFunctionStarted{
    my $self = shift;
    my ($Version, $Function) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "unable to register function, $Function is not valid" unless ($self->validFunction($Function));
    die "unable to register version, $Version is not valid" unless ($self->validVersion($Version));

    #check if patch task has an entry
    return 0 if (not exists $self->{'PatchSystemInfo'}->{'StartedPatchFunctions'}->{$Version}->{$Function});

    #get time
    my $TimeStamp = $self->{'PatchSystemInfo'}->{'StartedPatchFunctions'}->{$Version}->{$Function};

    return 1 if ($TimeStamp ne ''); #ROADWORKS check if it is a time date
    return 0;
}

sub existsCurrentFunctionVersion{
    my $self = shift;
    my ($Function) = @_;

    die "unable to set handler version, $Function is not valid" unless ($self->validFunction($Function));

    #get object from file
    $self->_collectSystemInfoFromFile();

    return exists $self->{'PatchSystemInfo'}->{'CurrentFunctionVersion'}->{$Function};
}

sub getCurrentHandlerVersion{
    my $self = shift;
    my ($Handler) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "unable to get handler version, $Handler is not valid" unless ($self->validHandler($Handler));
    die "PatchSystemInfo ERROR no entry for handler $Handler" unless (exists $self->{'PatchSystemInfo'}->{'CurrentHandlerVersion'}->{$Handler});

    return $self->{'PatchSystemInfo'}->{'CurrentHandlerVersion'}->{$Handler};
}

sub setCurrentHandlerVersion{
    my $self = shift;
    my ($Handler,$Version) = @_;

    die "unable to set handler version, $Handler is not valid" unless ($self->validHandler($Handler));
    die "unable to set handler version, $Version is not valid" unless ($self->validVersion($Version));

    $self->{'PatchSystemInfo'}->{'CurrentHandlerVersion'}->{$Handler} = $Version;

    $self->_saveToFile();

    return;
}

sub existsCurrentHandlerVersion{
    my $self = shift;
    my ($Handler) = @_;

    die "unable to check handler version, $Handler is not valid" unless ($self->validHandler($Handler));

    #get object from file
    $self->_collectSystemInfoFromFile();

    return (exists $self->{'PatchSystemInfo'}->{'CurrentHandlerVersion'}->{$Handler});
}

sub logPatchHistory{
    my $self = shift;
    my ($Version, $FunctionName, $Message, $LogLevel) = @_;

    die "unable to register function, $FunctionName is not valid" unless ($self->validFunction($FunctionName));
    die "unable to register version, $Version is not valid" unless ($self->validVersion($Version));

    #get ground function name
    my $GroundFunctionName = $FunctionName;
    $GroundFunctionName =~ s/^.*::(\w+)$/$1/;

    #clean function name
    $FunctionName =~ s/^API::Patch:://;

    #get handler name
    my $Handler = $FunctionName;
    $Handler =~s/::\Q$GroundFunctionName\E//;

    #get log level for ground function
    $LogLevel = $self->getGroundFunctionLogLevel($GroundFunctionName) unless (defined $LogLevel);

    my $TimeString = localtime;
    my $LogFile;
    my $i;

    #generate log line
    my $LogLine = "\n$TimeString | $Version | $FunctionName:  -  $Message";

    #from low important messages to high important messages, all messages get to the low important log file and only a few to higher priorisation
    for ($i=0;$i<=$LogLevel;$i++){

        #get file signed with level
        $LogFile = $self->getSystemInfo('PatchLogDir') . "/PatchLog-Level_$i.log";

        #write into log file
        WriteFile($LogFile, \$LogLine, {append => 1});
    }

    #get handler log file
    $LogFile = $self->getSystemInfo('PatchLogDir') . "/PatchLog-$Handler.log";

    #write into log file
    WriteFile($LogFile, \$LogLine, {append => 1});

    return;
}

sub versionLowerThan{
    my $self = shift;
    my ($Version, $CompareVersion) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "unable to register version, $Version is not valid" unless ($self->validVersion($Version));
    die "unable to register version, $CompareVersion is not valid" unless ($self->validVersion($CompareVersion));

    my $VersionPosition = IndexOf($Version, @{$self->validVersions()});
    my $CompareVersionPosition = IndexOf($CompareVersion, @{$self->validVersions()});

    return ($VersionPosition < $CompareVersionPosition);
}

sub versionGreaterThan{
    my $self = shift;
    my ($Version, $CompareVersion) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "unable to register version, $Version is not valid" unless ($self->validVersion($Version));
    die "unable to register version, $CompareVersion is not valid" unless ($self->validVersion($CompareVersion));

    my $VersionPosition = IndexOf($Version, @{$self->validVersions()});
    my $CompareVersionPosition = IndexOf($CompareVersion, @{$self->validVersions()});

    return ($VersionPosition > $CompareVersionPosition);
}

sub getCartridgesVersionFolder{
    my $self = shift;
    my ($Version) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "unable to get version folder, $Version is not valid" unless ($self->validVersion($Version));

    if (exists $self->{'PatchSystemInfo'}->{'CartridgesVersionFolder'}->{$Version}){
        return $self->{'PatchSystemInfo'}->{'CartridgesVersionFolder'}->{$Version};
    }
    else{
        return $self->getSystemInfo('EpagesCartridgesVersionDir') . "/$Version";
    }
}

sub setCartridgesVersionFolder{
    my $self = shift;
    my ($Version,$Directory) = @_;

    die "unable to set version folder, $Version is not valid" unless ($self->validVersion($Version));

    #get object from file
    $self->_collectSystemInfoFromFile();

    $self->{'PatchSystemInfo'}->{'CartridgesVersionFolder'}->{$Version} = $Directory;

    $self->_saveToFile();
}

sub getPatchVersionFolder{
    my $self = shift;
    my ($Version) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    die "unable to get version folder, $Version is not valid" unless ($self->validVersion($Version));

    return $self->getSystemInfo('PatchVersionDir') . "/$Version";
}

#ROADWORKS - docu return is an hash ref $hInstallStoreTypes ref.hash.array
sub getStoreTypes{
    my $self = shift;

    #get object from file
    $self->_collectSystemInfoFromFile();

    my @StoreTypes = keys %{$self->{'PatchSystemInfo'}->{'InstallStoreTypes'}};


    return \@StoreTypes;
}

sub getStoreTypeCartridges{
    my $self = shift;
    my ($StoreType) = @_;

    #get object from file
    $self->_collectSystemInfoFromFile();

    my @Cartridges;

    #return empty array if there is no entry for store type
    return \@Cartridges unless (
        exists $self->{'PatchSystemInfo'}->{'InstallStoreTypes'}->{$StoreType}
    );

    @Cartridges = @{$self->{'PatchSystemInfo'}->{'InstallStoreTypes'}->{$StoreType}};

    return \@Cartridges;

}

sub getProcessedHandlerPrognosis{
    my $self = shift;

    #get object from file
    $self->_collectSystemInfoFromFile();

    #all patch functions that count
    my @SpecificPatchFunction = ('doPatch', 'doPatch_basic', 'doPatch_universal', 'doPatch_checkPreCondition');
    my $AllPatchFunctions = scalar @{$self->validHandlers()} * scalar @SpecificPatchFunction;

    #count all tasks
    my $CountAllSteps = $AllPatchFunctions * ((scalar @{$self->validVersions()}) - 1);

    #placeholder for completed task
    my $CountCompletedTasks = 0;

    foreach my $Version (@{$self->validVersions()}){
        #look only for versions after the start version
        next unless ($self->versionGreaterThan($Version,$self->getSystemInfo('ePagesStartVersion')) );

        $CountCompletedTasks += scalar @{$self->getCompletedPatchFunctions($Version)};
    }

    return int(($CountCompletedTasks * 100) / $CountAllSteps);
}

sub getCompletedPatchFunctions{
    my $self = shift;
    my ($Version) = @_;

    die "unable to get completed patch functions, $Version is not valid" unless ($self->validVersion($Version));

    #get object from file
    $self->_collectSystemInfoFromFile();

    my @Result = keys %{$self->{'PatchSystemInfo'}->{'CompletedPatchFunctions'}->{$Version}};

    return \@Result;
}

sub setStartTime{
    my $self = shift;

    #get object from file
    $self->_collectSystemInfoFromFile();

    $self->{'PatchSystemInfo'}->{'PatchStatistic'}->{'StartTime'} = time unless(
        exists $self->{'PatchSystemInfo'}->{'PatchStatistic'}->{'StartTime'}
    );
}

sub getStartTime{
    my $self = shift;

    #get object from file
    $self->_collectSystemInfoFromFile();

    return $self->{'PatchSystemInfo'}->{'PatchStatistic'}->{'StartTime'};
}

sub getPastTimeMessage{
    my $self = shift;

    my $Time2 = time;
    my $Time1 = $self->getStartTime();

    my $Minutes = int ($Time2-$Time1) / 60;
    my $Seconds = int ($Time2-$Time1) % 60;
    my $Message = sprintf ('need %d:%02d minutes', $Minutes, $Seconds);

    #return past time
    return $Message;
}

#sub getWorkPrognosis{
#
#}

#sub getTimePrognosis{
#
#}

#sub getVersionsFrom{
#    my $self = shift;
#    my ($Version) = @_;
#
#    die "unable to get version array, $Version is not valid" unless ($self->validVersion($Version));
#
#    my $Found = 0;
#
#    foreach my $ValidVersion (@{$self->validVersions()}){
#
#    }
#
#}

1;