use strict;
use Test::More tests => 10;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (ExistsObjectByPath LoadRootObject LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::WebInterface::API::FakeRequest;

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

sub LoginToshop {
    my ($URL) = @_;

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

    my $UrlParams = join('&', (
        'ViewAction=View',
        'ChangeAction=SaveLoginForm',
        'Login=mmustermann',
        'Password=123456'
    ));

    my $Response = $FakeRequest->get(
        $URL . '&' . $UrlParams,
    );

    cmp_ok( $Response->code, '==', 200, 'SF login to shop response 200 OK' );

    # grep URL with sec-Token inside from content (LWP::UserAgent would do this aswell ...)
    # i.e.: <base href="https://qa-vm-lin64-1.intern.epages.de/epages/Store.admin/de_DE/">
    my ($SessionURL) = $Response->content =~ /<base\s*href=["'](.*?)["']/sx;   #'

    if(!$SessionURL || $SessionURL eq '') {
        # alternatively we have a javascript var for that URL
        # i.e.: epages.vars.BaseUrl = 'http://slange.intern.epages.de/epages/DemoShop.admin/sec7f9ebdacf6/';
        ($SessionURL) = $Response->content =~ /['"]baseUrl['"]\s*:\s*['"](.*?)['"]/sx;     #'
    }

    ok( $SessionURL =~ /Store\.sf\/sec/g, 'Search for session URL in content');

    return $SessionURL;
}

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;

    my $Shop = LoadObjectByPath('/Test/TestShop/');
    $Shop->set({
        InitialShopWizardStartup => 0,
        DomainName => 'www.myothertestshop.lan',
    });

    $Shop->featureMaxValue( 'Pages', 10 );

    # EPG-18427 : Pages with private customer data can be indexed by search engines
    # create test article page
    my $Art1 = $Shop->get('Categories')->insertChild('Article', { Alias => 'Article' });
    $Art1->set({
        'Name' => 'Nice test article - buy top test products!',
        'Description' => 'Curabitur pretium elementum purus, ac tincidunt tellus lobortis id. In quis blandit purus. Quisque quis elit varius est faucibus.',
        'Text' => 'Nunc arcu <em>metus</em>, dapibus in bibendum et, aliquet eu lectus. Donec ipsum tellus, tempor non dignissim at, vehicula lacinia enim. Morbi dignissim lacus eu ipsum placerat sit amet volutpat leo vestibulum.',
        'URI' => 'top-article-kat1',
    }, $Shop->get('LanguageID'));

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

    # accessing sf page without session has canonical tag but no robots tag
    my $ArticleUrl = 'http://www.myothertestshop.lan/top-article-kat1';
    my $Response = $FakeRequest->get($ArticleUrl);
    cmp_ok( $Response->code, '==', 200, 'Access article page via shorturl' );
    my $Content = $Response->decoded_content;
    ok($Content =~ /<link\s+rel=\"canonical\"\s+href=\"\Q$ArticleUrl\E\"\s+\/>/s, 'Canonical tag found');
    ok($Content !~ /<meta\s+name=\"robots\"/s, 'Robots meta tag not found');

    my $SessionURL = LoginToshop($Shop->get('WebUrlSSL'));


    # accessing sf page with sessionid in url in has no canonical tag but robots tag with noindex
    $Response = $FakeRequest->get($SessionURL.'?ObjectPath='.$Art1->get('Path'));

    #print $SessionURL.'?ObjectPath='.$Art1->get('Path');

    cmp_ok( $Response->code, '==', 200, 'Access article page with SessionID' );
    $Content = $Response->decoded_content;

    ok($Content !~ /<link\s+rel=\"canonical\"/s, 'Canonical tag not found');
    ok($Content =~ /<meta\s+name=\"robots\"\s+content=\"noindex,noarchive,nofollow\"/s, 'Noindex robots meta tag found');

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
