use Test::More tests => 40;

use strict;
use Config;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Image qw(
  AddBorderToImage
  Composite
  GetImageInfo
  NewImage
  ScaleImage
  ResizeImage
  TestImageFormat
  CorrectImage
  TestImage
  CreateIcon
  ResizeImageUsingImageRef
);
use DE_EPAGES::Core::API::File qw(
  CopyFile
  DeleteFile
  ExistsFile
  fsplit
  WriteFile
);
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );

use Image::Magick;
use Image::ExifTool;

## no critic qw(ProhibitMagicNumbers)

sub CompareImage {
    my ( $ImageFile1, $ImageFile2 ) = @_;

    if ( ExistsFile($ImageFile1) && ExistsFile($ImageFile2) ) {
        my $Image1 = Image::Magick->new;
        $Image1->Read($ImageFile1);
        my $Image2 = Image::Magick->new;
        $Image2->Read($ImageFile2);
        $Image1->Compare($Image2);

        # mean difference per pixel may not vary by more than this value
        # this may be caused by JPEG dithering
        return $Image1->Get('mean-error') <= 0.0002;
    }
    return 0;
}

my $TargetImageBase = 'Image/target';
my $HelperImage;
my $CompareImage;
my $WorkImage;
my $ImageHeight;
my $ImageWidth;
my $OrgImage;

RunScript( Sub => \&Main, Debug => 1 );

sub Main {

    my @SourceImages = ('Image/source.jpg');
    my $Counter      = 0;

    # test image formats, naming converntions, image content
    RunTestImageFormat( $SourceImages[0] );

    # test imagick ctor
    RunTestImage( $SourceImages[0] );

    foreach my $SourceImage (@SourceImages) {
        $Counter++;
        my ($Extension) = ( $SourceImage =~ /(\.[^\.]+)$/x );
        my $TargetImage = $TargetImageBase . $Extension;

        #AddBorderToImage - target bigger than source
        $CompareImage =
          'Image/compare_' . $Counter . '_AddBorderToImage1' . $Extension;
        CopyFile( $SourceImage, $TargetImage );
        AddBorderToImage( $TargetImage,
            { Width => 1100, Height => 337, BGColor => '#0066FF' } );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
            'AddBorderToImage - target bigger than source'
        );

        #AddBorderToImage - target smaller than source
        $CompareImage =
          'Image/compare_' . $Counter . '_AddBorderToImage2' . $Extension;
        CopyFile( $SourceImage, $TargetImage );
        AddBorderToImage( $TargetImage,
            { Width => 500, Height => 200, BGColor => '#0066FF' } );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
            'AddBorderToImage - target smaller than source'
        );

        #Composite - default gravity
        $CompareImage =
          'Image/compare_' . $Counter . '_Composite1' . $Extension;
        $HelperImage = 'Image/helper_Composite.jpg';
        CopyFile( $SourceImage, $TargetImage );
        Composite( $HelperImage, $TargetImage );
        ok( CompareImage( $TargetImage, $CompareImage ),
            'Composite - default gravity' );

        #Composite - North gravity
        $CompareImage =
          'Image/compare_' . $Counter . '_Composite2' . $Extension;
        CopyFile( $SourceImage, $TargetImage );
        Composite( $HelperImage, $TargetImage, { Gravity => 'North' } );
        ok( CompareImage( $TargetImage, $CompareImage ),
            'Composite - North gravity' );

        #Composite - NorthEast gravity
        $CompareImage =
          'Image/compare_' . $Counter . '_Composite3' . $Extension;
        CopyFile( $SourceImage, $TargetImage );
        Composite( $HelperImage, $TargetImage, { Gravity => 'NorthEast' } );
        ok( CompareImage( $TargetImage, $CompareImage ),
            'Composite - NorthEast gravity' );

        #Composite - East gravity
        $CompareImage =
          'Image/compare_' . $Counter . '_Composite4' . $Extension;
        CopyFile( $SourceImage, $TargetImage );
        Composite( $HelperImage, $TargetImage, { Gravity => 'East' } );
        ok( CompareImage( $TargetImage, $CompareImage ),
            'Composite - East gravity' );

        #Composite - SouthEast gravity
        $CompareImage =
          'Image/compare_' . $Counter . '_Composite5' . $Extension;
        CopyFile( $SourceImage, $TargetImage );
        Composite( $HelperImage, $TargetImage, { Gravity => 'SouthEast' } );
        ok( CompareImage( $TargetImage, $CompareImage ),
            'Composite - SouthEast gravity' );

        #Composite - South gravity
        $CompareImage =
          'Image/compare_' . $Counter . '_Composite6' . $Extension;
        CopyFile( $SourceImage, $TargetImage );
        Composite( $HelperImage, $TargetImage, { Gravity => 'South' } );
        ok( CompareImage( $TargetImage, $CompareImage ),
            'Composite - South gravity' );

        #Composite - SouthWest gravity
        $CompareImage =
          'Image/compare_' . $Counter . '_Composite7' . $Extension;
        CopyFile( $SourceImage, $TargetImage );
        Composite( $HelperImage, $TargetImage, { Gravity => 'SouthWest' } );
        ok( CompareImage( $TargetImage, $CompareImage ),
            'Composite - SouthWest gravity' );

        #Composite - NorthWest gravity
        $CompareImage =
          'Image/compare_' . $Counter . '_Composite8' . $Extension;
        CopyFile( $SourceImage, $TargetImage );
        Composite( $HelperImage, $TargetImage, { Gravity => 'NorthWest' } );
        ok( CompareImage( $TargetImage, $CompareImage ),
            'Composite - NorthWest gravity' );

        #Composite - Center gravity
        $CompareImage =
          'Image/compare_' . $Counter . '_Composite1' . $Extension;
        CopyFile( $SourceImage, $TargetImage );
        Composite( $HelperImage, $TargetImage, { Gravity => 'Center' } );
        ok( CompareImage( $TargetImage, $CompareImage ),
            'Composite - Center gravity' );

        #ScaleImage - max size for width OR height
        $CompareImage =
          'Image/compare_' . $Counter . '_ScaleImage1' . $Extension;
        ScaleImage( $SourceImage, $TargetImage, 150, 0 );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
            'ScaleImage - max size for width OR height'
        );

       #ScaleImage - max size for width OR height - checking default for param 4
        $CompareImage =
          'Image/compare_' . $Counter . '_ScaleImage2' . $Extension;
        ScaleImage( $SourceImage, $TargetImage, 150 );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
'ScaleImage - max size for width OR height - checking default for param 4'
        );

        #ScaleImage - max size for height only
        $CompareImage =
          'Image/compare_' . $Counter . '_ScaleImage3' . $Extension;
        ScaleImage( $SourceImage, $TargetImage, 100, 1 );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
            'ScaleImage - max size for height only'
        );

        #ScaleImage - max size for height only - with 50% quality
        $CompareImage =
          'Image/compare_' . $Counter . '_ScaleImage4' . $Extension;
        ScaleImage( $SourceImage, $TargetImage, 150, 1, 50 );
        ok( CompareImage( $TargetImage, $CompareImage ),
            'ScaleImage - max size for height only - with 50% quality' );



         #ResizeImageByImageRef - scale by width and height (width creates aspect ratio)
        $OrgImage = Image::Magick->new;
        $OrgImage->Read($SourceImage);

        $WorkImage = $OrgImage->Clone();

        $CompareImage =
          'Image/compare_' . $Counter . '_ResizeImage1' . $Extension;

        # get image width and height
        $ImageHeight = $OrgImage->Get('height');
        $ImageWidth  = $OrgImage->Get('width');

        # possible image magick cant read width/height
        if ( not defined $ImageWidth or not defined $ImageHeight ) {
            my $ImageInfo = ImageInfo($SourceImage);
            if (   defined( $ImageInfo->{'ImageHeight'} )
                && defined( $ImageInfo->{'ImageWidth'} ) )
            {

                # try to determine image dimensions via exif
                $ImageHeight = $ImageInfo->{'ImageHeight'};
                $ImageWidth  = $ImageInfo->{'ImageWidth'};
            }
        }
        ResizeImageUsingImageRef( $WorkImage, $SourceImage, $TargetImage,
            $ImageWidth, $ImageHeight, 250, 100 );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
'ResizeImageUsingImageRef - scale by width and height (width creates aspect ratio)'
        );

#ResizeImageByImageRef - scale by width and height (height creates aspect ratio)
        $OrgImage = Image::Magick->new;
        $OrgImage->Read($SourceImage);

        $WorkImage = $OrgImage->Clone();

        $CompareImage =
          'Image/compare_' . $Counter . '_ResizeImage2' . $Extension;

        # get image width and height
        $ImageHeight = $OrgImage->Get('height');
        $ImageWidth  = $OrgImage->Get('width');

        # possible image magick cant read width/height
        if ( not defined $ImageWidth or not defined $ImageHeight ) {
            my $ImageInfo = ImageInfo($SourceImage);
            if (   defined( $ImageInfo->{'ImageHeight'} )
                && defined( $ImageInfo->{'ImageWidth'} ) )
            {

                # try to determine image dimensions via exif
                $ImageHeight = $ImageInfo->{'ImageHeight'};
                $ImageWidth  = $ImageInfo->{'ImageWidth'};
            }
        }
        ResizeImageUsingImageRef( $WorkImage, $SourceImage, $TargetImage,
            $ImageWidth, $ImageHeight, 250, 50 );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
'ResizeImageUsingImageRef - scale by width and height (height creates aspect ratio)'
        );

        #ResizeImageByImageRef - max size for height only - 50% quality
        $OrgImage = Image::Magick->new;
        $OrgImage->Read($SourceImage);

        $WorkImage = $OrgImage->Clone();

        $CompareImage =
          'Image/compare_' . $Counter . '_ResizeImage3' . $Extension;

        # get image width and height
        $ImageHeight = $OrgImage->Get('height');
        $ImageWidth  = $OrgImage->Get('width');

        # possible image magick cant read width/height
        if ( not defined $ImageWidth or not defined $ImageHeight ) {
            my $ImageInfo = ImageInfo($SourceImage);
            if (   defined( $ImageInfo->{'ImageHeight'} )
                && defined( $ImageInfo->{'ImageWidth'} ) )
            {

                # try to determine image dimensions via exif
                $ImageHeight = $ImageInfo->{'ImageHeight'};
                $ImageWidth  = $ImageInfo->{'ImageWidth'};
            }
        }

        ResizeImageUsingImageRef( $WorkImage, $SourceImage, $TargetImage,
            $ImageWidth, $ImageHeight, 250, 100, 50 );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
            'ResizeImageUsingImageRef - max size for height only - 50% quality'
        );

        #ResizeImage - scale by width and height (width creates aspect ratio)
        $CompareImage =
          'Image/compare_' . $Counter . '_ResizeImage1' . $Extension;
        ResizeImage( $SourceImage, $TargetImage, 250, 100 );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
'ResizeImage - scale by width and height (width creates aspect ratio)'
        );

        #ResizeImage - scale by width and height (height creates aspect ratio)
        $CompareImage =
          'Image/compare_' . $Counter . '_ResizeImage2' . $Extension;
        ResizeImage( $SourceImage, $TargetImage, 250, 50 );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
'ResizeImage - scale by width and height (height creates aspect ratio)'
        );

        #ResizeImage - max size for height only - 50% quality
        $CompareImage =
          'Image/compare_' . $Counter . '_ResizeImage3' . $Extension;
        ResizeImage( $SourceImage, $TargetImage, 250, 100, 50 );
        ok(
            CompareImage( $TargetImage, $CompareImage ),
            'ResizeImage - max size for height only - 50% quality'
        );
    }

    my $TargetImage = $TargetImageBase . '.jpg';
    my $SourceImage = 'Image/source.jpg';

    #CreateIcon
    my $Icon = 'Image/target.ico';
    CreateIcon( $SourceImage, $Icon );
    ok( -e $Icon, 'CreateIcon - ico created' );

    #GetImageInfo
    my $compareResult = GetImageInfo( $SourceImage, 1 );
    ok( $compareResult->{'Width'} == 1000, 'GetImageInfo - Width' );
    ok( $compareResult->{'Height'} == 237, 'GetImageInfo - Height' );
    ok( $compareResult->{'Mimetype'} eq 'image/jpeg',
        'GetImageInfo - Mimetype' );
    ok(
        ''
          . $compareResult->{'Exif'}->{'ProfileDateTime'} eq
          '1998-02-09T06:49:00',
        'GetImageInfo - Exif (ProfileDateTime)'
    );

    #NewImage - JPG
    $CompareImage = 'Image/compare_NewImage1.jpg';
    NewImage( 'Image/target.jpg',
        { Width => 100, Height => 100, BGColor => '#0066FF' } );
    ok( CompareImage( 'Image/target.jpg', $CompareImage ), 'NewImage - JPG' );

    #NewImage - GIF
    $CompareImage = 'Image/compare_NewImage2.gif';
    my $TargetImage2 = 'Image/target.gif';
    NewImage( $TargetImage2,
        { Width => 100, Height => 100, BGColor => '#0066FF' } );
    ok( CompareImage( 'Image/target.gif', $CompareImage ), 'NewImage - GIF' );

    # cleanup
    DeleteFile('Image/target.ico');
    DeleteFile('Image/target.jpg');
    DeleteFile('Image/target.gif');
    DeleteFile('Image/target.png');

    # check png resizing should keep bit depth
    my $Png8Bits   = 'Image/png8bit.png';
    my $TempTarget = 'Image/tmptarget.png';
    ScaleImage( $Png8Bits, $TempTarget, 200, 0 );
    my $TempImage = Image::Magick->new;
    $TempImage->Read($TempTarget);
    cmp_ok( $TempImage->Get('type'),
        'eq', 'Palette', 'ScaleImage - Resized 8-bit PNG remains 8-bits' );
    undef $TempImage;
    DeleteFile($TempTarget);

    my $Png8BitsAlpha = 'Image/png8bit_alpha.png';
    ScaleImage( $Png8BitsAlpha, $TempTarget, 200, 0 );
    $TempImage = Image::Magick->new;
    $TempImage->Read($TempTarget);
    cmp_ok( $TempImage->Get('type'), 'eq', 'TrueColorMatte',
'ScaleImage - Resized 8-bit with alpha PNG is converted to 32-bit to look nicer'
    );
    undef $TempImage;
    DeleteFile($TempTarget);

    my $Png24Bits = 'Image/png24bit.png';
    ScaleImage( $Png24Bits, $TempTarget, 200, 0 );
    $TempImage = Image::Magick->new;
    $TempImage->Read($TempTarget);
    cmp_ok( $TempImage->Get('type'),
        'eq', 'TrueColor', 'ScaleImage - Resized 24-bit PNG remains 24-bits' );
    undef $TempImage;
    DeleteFile($TempTarget);

    my $Png32Bits = 'Image/png32bit.png';
    ScaleImage( $Png32Bits, $TempTarget, 200, 0 );
    $TempImage = Image::Magick->new;
    $TempImage->Read($TempTarget);
    cmp_ok( $TempImage->Get('type'),
        'eq', 'TrueColorMatte',
        'ScaleImage - Resized 32-bit PNG remains 32-bits' );
    undef $TempImage;
    DeleteFile($TempTarget);

    # check color profiles removal
    my $ProfileImage = 'Image/colorprofiles.jpg';
    my $ET           = Image::ExifTool->new;
    $ET->ImageInfo($ProfileImage);
    my @TagList = $ET->GetFoundTags();
    ok( ( 'ProfileCMMType' ~~ @TagList ) && ( 'ColorSpaceData' ~~ @TagList ),
        'CorrectImage - Org image contains color profiles' );
    my $ProfileTarget = 'Image/colorprofiles_tmptarget.jpg';
    CopyFile( $ProfileImage, $ProfileTarget );
    CorrectImage($ProfileTarget);
    $ET->ImageInfo($ProfileTarget);
    @TagList = $ET->GetFoundTags();
    ok(
        !( 'ProfileCMMType' ~~ @TagList ) && !( 'ColorSpaceData' ~~ @TagList ),
        'CorrectImage - Corrected image does not contain color profiles'
    );
    DeleteFile($ProfileTarget);

    # check color space conversion
    my $CMYKImage = 'Image/cmyk.jpg';
    $TargetImage = 'Image/cmyk_tmptarget.jpg';
    $TempImage   = Image::Magick->new;
    $TempImage->Read($CMYKImage);
    cmp_ok( $TempImage->Get('colorspace'),
        'eq', 'CMYK', 'CorrectImage - Org image is CMYK' );
    undef $TempImage;
    CopyFile( $CMYKImage, $TargetImage );
    CorrectImage($TargetImage);

    $TempImage = Image::Magick->new;
    $TempImage->Read($TargetImage);
    cmp_ok( $TempImage->Get('colorspace'),
        'eq', 'RGB', 'CorrectImage - Corrected image is RGB' );
    DeleteFile($TargetImage);
}

sub RunTestImageFormat {
    my ($ImageSourceFile) = @_;
    my ( undef, undef, $Ext ) = fsplit($ImageSourceFile);

    my $TestName =  'Image/' . '#+~\'`!$%&()={[]}' . $Ext;    ## no critic qw(RequireInterpolationOfMetachars)
    CopyFile( $ImageSourceFile, $TestName );
    TestImageFormat( 'Test', $TestName );
    ok( !ExistsError, 'TestImageFormat crypt name' );
    DeleteFile($TestName);
}

sub RunTestImage {
    my ($ImageSourceFile) = @_;
    eval { TestImage($ImageSourceFile); };
    ok( !ExistsError(), 'TestImage ok with valid image' );

    eval { TestImage( $ImageSourceFile . '_asdfjkhasfhaf.jpg' ); };
    ok( ExistsError() && GetError()->code eq 'FILE_NOT_FOUND',
        'TestImage error if file not exists' );

    my $Tmp = 'source_asdfjkhasfhaf.jpg';
    my $Content = 0xFFC1000C;    ## no critic qw(ProhibitMagicNumbers)
    WriteFile( $Tmp, \$Content, { 'binary' => 1 } );
    eval {                       ## no critic qw(RequireExistsError)
        TestImage($Tmp);
    };
    DeleteFile($Tmp);
    my $Error = GetError();
    ok(
        ExistsError() && $Error->code eq 'INVALID_IMAGE_FILE'
          || $Error->code qw 'BUG_EPG-18210_OCCURED',
        'TestImage error if broken image'
    );
}
