use Test::More tests => 52;
use strict;
use Config;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Core::API::Script qw( RunScript );

RunScript( Sub => sub {

    my $iniFile = './Ini/epages.conf';
    my $iniFile2 = './Ini/Database.conf';

    # Initalization
    eval {
        DE_EPAGES::Core::API::IniConfig->new( );
    };
    ok( ExistsError(), 'Filename or create missing in constructor' );
    is( GetError->code, 'NoParameter', 'error code: NoParameter' );

    eval {
        DE_EPAGES::Core::API::IniConfig->new( Create => 1);
    };
    ok( ExistsError(), 'Non-existing filename in constructor' );
    is( GetError->code, 'NoParameter', 'error code: NoParameter' );

    eval {
        DE_EPAGES::Core::API::IniConfig->new( FileName => 'nofile.ini' );
    };
    ok( ExistsError(), 'Filename missing in constructor' );
    is( GetError->code, 'NoFile', 'error code: NoFile' );

    eval {
        DE_EPAGES::Core::API::IniConfig->new( FileName => './Ini/Invalid.conf' );
    };
    ok( ExistsError(), 'Non-existing filename in constructor' );
    is( GetError->code, 'OpenRead', 'error code: OpenRead' );

    eval {
        my $iniF2 = './Ini/epages2.conf';
        DE_EPAGES::Core::API::IniConfig->new( FileName => $iniF2, Create => 1 );
    };
    ok (! ExistsError(), 'Creating new config file');


    my $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile );
    ok($ini, 'Given filename should be used as config file');

    my $ini2 = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile2 );
    ok($ini2->existsSection('Store.Default'), 'Reinit successfull');
    ok($ini->existsSection('DE_EPAGES::Mail'), 'Reinit successfull');

    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile );

    # get
    ok( $ini->get('DE_EPAGES::Store', 'DefaultWebServicePassword') eq 'wsadmin', 'Getting value from ini file');
    ok( $ini->get('de_epages::Store', 'DefaultWebServicePassword') eq 'wsadmin', 'Getting value from ini file - section in lowercase');
    ok( $ini->get('DE_EPAGES::Store', 'defaultwebservicepassword') eq 'wsadmin', 'Getting value from ini file - parameter in lowercase');
    ok( $ini->get('Foo:bar', 'FUBAR', 'SNAFU') eq 'SNAFU', 'Getting default value from ini file');
    ok( $ini->get('Foo:bar', 'fubar', 'SNAFU') eq 'SNAFU', 'Getting default value from ini file - in lowercase');
    ok( ! $ini->get('Foo:bar', 'FUBAR'), 'No value found in ini file');

    # set and check if changes are written to file
    ok( $ini->set ('DE_EPAGES::Store', 'DefaultWebServicePassword', 'wsadmin2') , 'Setting existing parameter on existing section');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile );
    ok( $ini->get('DE_EPAGES::Store', 'DefaultWebServicePassword') eq 'wsadmin2', 'Check setting existing parameter on existing section');
    $ini->set ('DE_EPAGES::Store', 'DefaultWebServicePassword', 'wsadmin');

    ok ($ini->set ('DE_EPAGES::Store', 'Testkey1', 'Testvalue1'), 'Setting new parameter on existing section');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile );
    ok( $ini->get('DE_EPAGES::Store', 'Testkey1') eq 'Testvalue1', 'Check setting new parameter on existing section');

    ok( $ini->set ('Foo:bar', 'Testkey1', 'Testvalue1'), 'Setting new parameter on new section');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile );
    ok( $ini->get('Foo:bar', 'Testkey1') eq 'Testvalue1', 'Check setting new parameter on new section');
    $ini->delete('Foo:bar', 'Testkey1');
    $ini->rewrite();

    # delete key
    ok( defined $ini->get('DE_EPAGES::Store', 'Testkey1'), 'before deleting parameter');
    $ini->delete('DE_EPAGES::Store', 'Testkey1');
    ok( !defined $ini->get('DE_EPAGES::Store', 'Testkey1'), 'Deleting parameter');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile );
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), undef, 'Check Deleting parameter');
    ok( ! exists $ini->section('DE_EPAGES::Store')->{'testkey1'}, 'Check Deleting parameter 1');
    ok( ! exists $ini->section('DE_EPAGES::Store')->{'Testkey1'}, 'Check Deleting parameter 2');

    # empty key
    $ini->set('DE_EPAGES::Store', 'Testkey1', '' );
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), '', 'Set key to empty string');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile );
    ok( exists $ini->section('DE_EPAGES::Store')->{'testkey1'}, 'Check  Set key to empty string 1');
    ok( !exists $ini->section('DE_EPAGES::Store')->{'Testkey1'}, 'Check  Set key to empty string 2');
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), '', 'Check Set key to empty string 3');

    # undefined key
    $ini->set('DE_EPAGES::Store', 'Testkey1', undef );
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), '', 'Set key to undef');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile );
    ok( exists $ini->section('DE_EPAGES::Store')->{'testkey1'}, 'Check  Set key to undef 1');
    ok( !exists $ini->section('DE_EPAGES::Store')->{'Testkey1'}, 'Check  Set key to undef 2');
    is( $ini->get('DE_EPAGES::Store', 'Testkey1'), '', 'Check Set key to undef 3');

    # check existsSection
    ok( $ini->existsSection('DE_EPAGES::Store'), 'Checking existing section');
    ok( ! $ini->existsSection('Bla::Blub'), 'Checking nonexisting section');

    # add section
    ok( $ini->addSection('Bla:blubb') eq 'Bla:blubb', 'Adding section');
    ok( $ini->existsSection('Bla:blubb'), 'Check adding section');

    # get section names
    my $aSections = $ini->sectionNames ('DE_EPAGES::Store');
    ok( scalar @$aSections  == 7, 'Getting correct number of section names');

    $aSections = $ini->sectionNames ('de_epages::store');
    ok( scalar @$aSections  == 7, 'Getting correct number of section names - with lowercase');



    # adding and returning section values
    my $hSection = $ini->section('DE_EPAGES::Store');
    ok ($$hSection{lc 'DefaultWebServicePassword'} eq 'wsadmin' , 'Getting hash from section');
    # get keys case sensitive
    my $ini3 = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile, CaseSensitive => 1 );
    my $hSection3 = $ini3->section('DE_EPAGES::Store');
    ok ($$hSection3{'DefaultWebServicePassword'} eq 'wsadmin' , 'Getting hash from section case sensitive');
    ok (! $hSection3->{lc 'DefaultWebServicePassword'}, 'Getting no hash from section case sensitive');


    $hSection = {'newkey2' => 'newval2'};
    $ini->section('DE_EPAGES::Store', $hSection);
    my $hSection2 = $ini->section('DE_EPAGES::Store');
    ok ($hSection2->{'newkey2'} eq $$hSection{'newkey2'}, 'Getting new inserted value back in hash from section');

    # deleting section
    $ini->deleteSection('Bla:blubb');
    $ini->deleteSection('Foo:bar');
    $ini->rewrite();
    $ini = DE_EPAGES::Core::API::IniConfig->new( FileName => $iniFile );
    ok( ! $ini->existsSection('Foo:bar'), 'Check deleting section');

    # get content
    #print $ini->content() ;
    my $content = $ini->content();
    ok( $content, 'Content of ini file');

    my @comments = $content =~ /(#\w+)/g;
    ok(scalar @comments > 0, 'Content has some comments');


    # creating new file

    my $ini4 = DE_EPAGES::Core::API::IniConfig->new( FileName => 'test2.ini', Create => 1 , CaseSensitive => 1);
    ok( $ini4->set ('Foo:bar', 'Testkey1', 'Testvalue1'), 'Setting new parameter on new file');
    $ini4->rewrite();
    $ini4 = DE_EPAGES::Core::API::IniConfig->new( FileName => 'test2.ini', CaseSensitive => 1 );
    ok( $ini4->get('Foo:bar', 'Testkey1') eq 'Testvalue1', 'Check setting new parameter on new file');

    unlink('test2.ini');
});
