use strict;
use Test::More tests => 52;
use Scalar::Util qw(blessed);
use DE_EPAGES::Core::API::Object::NumberFormat;
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);

## no critic qw( ProhibitLongLines ProhibitMagicNumbers RequireInterpolationOfMetachars)

my $en_Number = DE_EPAGES::Core::API::Object::NumberFormat->new(
    decimal_point   => '.',
    thousands_sep   => ',',
    decimal_fill    => 0,
    neg_format      => '-x',
);

my $en_USD = DE_EPAGES::Core::API::Object::NumberFormat->new(
    mon_decimal_point  => '.',
    mon_thousands_sep  => ',',
    decimal_point      => '.',
    thousands_sep      => ',',
    int_curr_symbol    => '$',
    decimal_fill       => 1,
    neg_format         => '-x',
    int_currencyid     => 'USD',
    curr_pos_format    => 'x',
    curr_neg_format    => '-x',
    decimal_digits => 2,
);

my $de_EUR = DE_EPAGES::Core::API::Object::NumberFormat->new(
    mon_decimal_point  => ',',
    mon_thousands_sep  => '.',
    decimal_point      => ',',
    thousands_sep      => '.',
    int_curr_symbol => '',
    decimal_fill => 1,
    curr_pos_format => 'x ',
    curr_neg_format => '-x ',
    neg_format => '-x',
    int_currencyid => 'EUR',
    decimal_digits => 2,
);

my $fr_CAN = DE_EPAGES::Core::API::Object::NumberFormat->new(
    mon_decimal_point  => '.',
    mon_thousands_sep  => ',',
    decimal_point      => '.',
    thousands_sep      => ',',
    int_curr_symbol => '$',
    decimal_fill => 1,
    curr_pos_format => 'x',
    curr_neg_format => 'x-',
    neg_format => '-x',
    int_currencyid => 'CAN',
    decimal_digits => 2,
);

my $ch_CH = DE_EPAGES::Core::API::Object::NumberFormat->new(
    mon_decimal_point  => '.',
    mon_thousands_sep  => ',',
    decimal_point      => '.',
    thousands_sep      => ',',
    int_curr_symbol => '$',
    decimal_fill => 1,
    curr_pos_format => 'x',
    curr_neg_format => 'x-',
    neg_format => '-x',
    int_currencyid => 'CHF',
    decimal_digits => 2,
);

my $SymbolXPF = '\x{8355}'; utf8::upgrade($SymbolXPF);
my $fr_XPF = DE_EPAGES::Core::API::Object::NumberFormat->new(
    mon_decimal_point  => ',',
    mon_thousands_sep  => '.',
    decimal_point      => ',',
    thousands_sep      => '.',
    int_curr_symbol => $SymbolXPF,
    decimal_fill => 1,
    curr_pos_format => 'x ',
    curr_neg_format => '-x ',
    neg_format => '-x',
    int_currencyid => 'XPF',
    decimal_digits => 0,
);

my $fr_TND = DE_EPAGES::Core::API::Object::NumberFormat->new(
    mon_decimal_point  => ',',
    mon_thousands_sep  => '.',
    decimal_point      => ',',
    thousands_sep      => '.',
    int_curr_symbol => 'D',
    decimal_fill => 1,
    curr_pos_format => 'x ',
    curr_neg_format => '-x ',
    neg_format => '-x',
    int_currencyid => 'TND',
    decimal_digits => 3,
);

my $numberF = DE_EPAGES::Core::API::Object::NumberFormat->new(
        decimal_point  => ',',
        thousands_sep  => '.',
        neg_format     => '-x',
        decimal_digits => 0,
    );

my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new;
ok( !blessed( $Formatter->round(0.5) ), 'not an object');
is( $Formatter->round(0.5), '0.5', 'round(0.5)' );
is( $Formatter->round(0), '0', 'round(0)' );
is( $Formatter->round(1.0), '1', 'round(1.0)' );
is( $Formatter->round(1E20), '100000000000000000000', 'round(1E20)' );

ok( $en_Number->round(999999999.123456789) eq '999999999.123457', 'round 999999999.123456789 is ' . $en_Number->round(999999999.123456789));
ok( $en_Number->round(123456789.12345678) eq '123456789.123457', 'round 123456789.12345678 is ' . $en_Number->round(123456789.123457));
ok( $en_Number->round(12.123456789163456789,10) eq '12.1234567892', 'round (12.123456789163456789,10) is '. $en_Number->round(12.123456789163456789,10));
ok( $en_Number->round(100000000000000) eq '100000000000000', 'round 100000000000000 is ' . $en_Number->round(100000000000000));
ok( $en_Number->round(1000) eq '1000', 'round 10000 is ' . $en_Number->round(1000));
ok( $ch_CH->roundTotal(999999999999.99, 6) eq '999999999999.99', 'roundTotal(999999999999.99, 6) is 999999999999.99');
ok( $ch_CH->roundTotal(-99999999999.999, 6) eq '-99999999999.999', 'roundTotal(-99999999999.999, 6) is -99999999999.999');
# known "overflow"
ok( $ch_CH->roundTotal(99999999999999.9999) eq '100000000000000', 'roundTotal(99999999999999.9999) is 100000000000000');

ok( $en_Number->format_number(12345) eq '12,345', 'format 12,345 is ' . $en_Number->format_number(12345));
ok( $en_Number->format_number(-12345) eq '-12,345', 'format -12,345 is ' . $en_Number->format_number(-12345));
ok( $en_Number->format_number(-12345.123456789,10) eq '-12,345.123456789', 'format(-12,345.123456789,10) is ' . $en_Number->format_number(-12345.123456789,10));
ok( $en_Number->format_number(-12345.12345678909,10) eq '-12,345.1234567891', 'format(-12,345.12345678909,10) is ' . $en_Number->format_number(-12345.12345678909,10));
ok( $en_Number->format_number(-12345.12345678909, 3) eq '-12,345.123', 'format -12,345.123, 3 is ' . $en_Number->format_number(-12345.123, 3));

ok( $en_USD->format_price(1234567) eq '$1,234,567.00', 'format $ 1,234,567.00 is ' . $en_USD->format_price(1234567));
ok( $en_USD->format_price(-123) eq '-$123.00', 'format $ -123.00 is '. $en_USD->format_price(-123));
ok( $en_USD->unformat_number('$12,345.67') == 12345.67, 'parse $12,345.67');
ok( $en_USD->unformat_number('-$123.00') == -123, 'parse -$123.00');
ok( $en_Number->unformat_number('12,345.67') == 12345.67, 'parse 12,345.67');
ok( $de_EUR->format_price(1234567) eq '1.234.567,00 ', 'format 1.234.567,00  is ' . $de_EUR->format_price(1234567));
ok( $de_EUR->format_price(-123) eq '-123,00 ', 'format -123,00  is '. $de_EUR->format_price(-123));

# this may change when EPG-7959 is resolved
ok( $de_EUR->format_price(31.3888888888889) eq '31,389 ', 'see EPG-7959 '. $de_EUR->format_price(31.3888888888889));

ok( $de_EUR->unformat_number('1.234.567,00 ') == 1234567 , 'parse 1.234.567,00 ');
ok( $de_EUR->unformat_number('- 123,00 ') == -123, 'parse -123,00 ');
ok( $de_EUR->unformat_number('-  123,00') == -123, 'parse -  123,00');
ok( $fr_CAN->unformat_number('123.00$-') == -123, 'parse 123.00$-');
ok( $fr_CAN->unformat_number('123.00 - $') == -123, 'parse 123.00$-');
ok( $fr_CAN->unformat_number('123$00 -') == -123, 'parse 123$00-');

# currency with 0 digits
ok( $fr_XPF->format_number(-12.345) eq '-12', 'format -12.345 (XPF)');
my $XPF = '\x{8355}'; utf8::upgrade($XPF);
ok( $fr_XPF->format_price(-12.345) eq "-12,3 $XPF", 'format -12.345 XPF');
ok( $fr_XPF->format_price(-12.0) eq "-12 $XPF", 'format -12.0 XPF');
ok( $fr_XPF->format_price(-12.01) eq "-12 $XPF", 'format -12.01 XPF');
ok( $fr_XPF->unformat_number("-12 $XPF") == -12, 'parse -12 XPF');
ok( $fr_XPF->unformat_number("-12,3 $XPF") == -12.3, 'parse -12,3 XPF');

# currency with 3 digits
ok( $fr_TND->format_number(-12.345) eq '-12,345', 'format- 12.345 (TND)');
ok( $fr_TND->format_number(-12.34567) eq '-12,346', 'format -12.34567 (TND)');
ok( $fr_TND->format_price(-12.34567) eq '-12,3457 D', 'format -12.345 TND');
ok( $fr_TND->format_price(-12.3) eq '-12,300 D', 'format -12.3 TND');

ok( $fr_TND->unformat_number('-12,345 D') == -12.345, 'parse TND');

is( $ch_CH->format_number(123.49), 123.49, 'format CHF ' . $ch_CH->format_number(123.49));
is( $ch_CH->format_number(123.50), '123.50', 'format CHF ' . $ch_CH->format_number(123.50));
is( $ch_CH->round(123.49), 123.49, 'round CHF ' . $ch_CH->round(123.49));
is( $ch_CH->roundTotal(123.49), 123.50, 'roundTotal CHF ' . $ch_CH->roundTotal(123.49));

eval {
    ok( $ch_CH->unformat_number('12ad.50') == 123.50, 'unformat_number 12ad.50');
};
ok( (ExistsError() and GetError()->code eq 'FORMAT_NOT_INTEGER'), 'unformat_number 12ad.50');
eval {
    ok( $ch_CH->unformat_number('12.50bc') == 123.50, 'unformat_number 12.50bc');
};
ok( (ExistsError() and GetError()->code eq 'FORMAT_NOT_INTEGER'), 'unformat_number 12.50bc');
eval {
    ok( $en_Number->unformat_number('-$123.00') == 123.50, 'unformat_number -$123.00');
};
ok( (ExistsError() and GetError()->code eq 'FORMAT_NOT_INTEGER'), 'unformat_number -$123.00');
ok( $numberF->format_number('1234567.89') eq '1.234.568', 'bug 2.351');


my $Price = 0;
my $RoundPrice = $de_EUR->round($Price);
ok ( $de_EUR->compare($Price, $RoundPrice) == 0, 'compare 0');
