use strict;

use lib (
    $ENV{EPAGES_CARTRIDGES}.'/DE_EPAGES/Core/Scripts',
    $ENV{EPAGES_CARTRIDGES}.'/DE_EPAGES/XML/Scripts');

use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use Perl::Critic;
use PPI::Document;
use PPI::Token::Comment;
use Test::More;

sub Main {
    testManual();
    testChangeQuotes();
    testCorrectArguments();
    testNoCritic();
    testOrganizeImports();
    testCorrectStringySplit();
    testCorrectDoubleSigils();
    testChangeBooleanGrep();
    testAddTrailingComma();
    testCorrectIndirectSyntax();
    done_testing();
}

sub testManual {
    my $Document = PPI::Document->new( \"print 'Hello World';" );
    isa_ok( $Document, 'PPI::Document' );
    my $string = $Document->find_first('Token::Word');
    isa_ok( $string, 'PPI::Token::Word' );
    is( $string->content, 'print', 'Got expected token' );
    my $Statement = $string->statement;
    isa_ok($Statement, 'PPI::Statement');
    my $semicolon = $Statement->last_token;
    isa_ok($semicolon, 'PPI::Token::Structure');
    my $foo = PPI::Token::Comment->new(' # foo');
    isa_ok( $foo, 'PPI::Token::Comment' );
    is( $foo->content, ' # foo', 'Created Comment token' );
    $Statement->insert_after( $foo );
    is( $Document->serialize, 'print \'Hello World\'; # foo',
    'insert_after actually inserts' );
}

sub getCritic {
    my $critic = Perl::Critic->new(
        -profile  => 'NONE',
        -severity => 1
    );
    ok( defined $critic, 'new Perl::Critic instance created' );
    return $critic;

}

sub testNoCritic { ## no critic (DE_EPAGES::Style::ProhibitFunctionSize)
    my $critic = getCritic();
    $critic->add_policy( -policy => 'Perl::Critic::Policy::DE_EPAGES::Variables::ProhibitUnusedVariablesEnhanced' );

    ##no critic qw( ProhibitImplicitNewlines )
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::NoCritic', q(
    use strict;
        sub x {
            my $x = '10';
            my $y = 0;
        }
    ), q(
    use strict;
        sub x {
            my $x = '10'; ## no critic (ProhibitUnusedVariablesEnhanced)
            my $y = 0; ## no critic (ProhibitUnusedVariablesEnhanced)
        }
    ), 2, 'Two violations');
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::NoCritic', q(
        sub x {
            my ($x, $y);
        }
    ), q(
        sub x {
            my ($x, $y); ## no critic (ProhibitUnusedVariablesEnhanced)
        }
    ), 2, "Two on one line");
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::NoCritic', q(
        sub x {
            my $x = '10'; $y = 20;
            $y++;
        }
    ), q(
        sub x {
            my $x = '10'; ## no critic (ProhibitUnusedVariablesEnhanced)
            $y = 20;
            $y++;
        }
    ), 1, "Line continues");
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::NoCritic', q(
        sub x {
            foreach my $x (1..10) {
                print "test\n";
            }
        }
    ), q(
        sub x {
            foreach my $x (1..10) {
                print "test\n";
            }
        }
    ), 0, "No violation");
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::NoCritic', q(
        sub x { my $x = '10'; $y = 20; $y++; }
    ), q(
        sub x { my $x = '10'; ## no critic (ProhibitUnusedVariablesEnhanced)
                $y = 20; $y++; }
    ), 1, "no linebreaks");
}

sub testChangeQuotes { ## no critic (DE_EPAGES::Style::ProhibitFunctionSize)
    my $critic = getCritic();
    $critic->add_policy( -policy => 'Perl::Critic::Policy::ValuesAndExpressions::ProhibitInterpolationOfLiterals' );
    ##no critic qw( ProhibitImplicitNewlines )
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::ChangeQuotes', q(
        sub x { 
            my $x = "Test"; 
            my $y = 'ABC';
        }
    ), q(
        sub x { 
            my $x = 'Test'; 
            my $y = 'ABC';
        }
    ), 1, "simple replace");
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::ChangeQuotes', q(
        sub x { 
            my $x = "Test
            asdfasdf $a
            "; 
            my $y = "
            Test \"
            ";
        }
    ), q(
        sub x { 
            my $x = "Test
            asdfasdf $a
            "; 
            my $y = '
            Test "
            ';
        }
    ), 1, "newlines and escapes");

    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::ChangeQuotes', q{
        sub x {
            my $Document = PPI::Document->new( \"print Hello World;" );
        }
    }, q{
        sub x {
            my $Document = PPI::Document->new( \"print Hello World;" );
        }
    }, 1, 'string reference');

    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::ChangeQuotes', q{
        sub x {
            is( $Document->serialize, "print 'Hello World'; # foo",
        }
    }, q{
        sub x {
            is( $Document->serialize, 'print \'Hello World\'; # foo',
        }
    }, 1, 'escape single quotes.');
}

sub testCorrectArguments {
    my $critic = getCritic();
    $critic->add_policy( -policy => 'Perl::Critic::Policy::DE_EPAGES::Style::RequireArgUnpacking' );
    ##no critic qw( ProhibitImplicitNewlines )
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::CorrectArguments', q(
        sub x {
            my $a = shift;
            my ($b, $c) = @_;
            return $a + $b;
        }
    ), q(
        sub x {
            my ($a, $b, $c) = @_;
            return $a + $b;
        }
    ), 1, "correct arguments");

#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::CorrectArguments', q(
sub InsertAccessory {
    my $self = shift;
    my $dbi = shift;
    my ($rhParameter) = @_;
    $dbi->do('INSERT INTO accessory (accessoryid, productid, targetproductid) VALUES (?, ?, ?)',
             [ @{$rhParameter}{qw( AccessoryID ProductID TargetProductID )} ]
    );
}
    ), q(
sub InsertAccessory {
    my $self = shift;
    my ($dbi, $rhParameter) = @_;
    $dbi->do('INSERT INTO accessory (accessoryid, productid, targetproductid) VALUES (?, ?, ?)',
             [ @{$rhParameter}{qw( AccessoryID ProductID TargetProductID )} ]
    );
}
    ), 1, "correct arguments 2");

}

sub testOrganizeImports { ## no critic (DE_EPAGES::Style::ProhibitFunctionSize)
    my $critic = getCritic();
    $critic->add_policy( -policy => 'Perl::Critic::Policy::DE_EPAGES::Style::RequireCorrectImports' );
    ##no critic qw( ProhibitImplicitNewlines )
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::OrganizeImports', q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Object::API::Factory qw ( Transaction );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::PerlTools qw ( Compare );
use SOAP::Lite;

sub x {
    my $self = shift;
    Transaction(Compare($self, ElementOf($self)));
}

1;
), q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( Compare ElementOf );
use DE_EPAGES::Object::API::Factory qw ( Transaction );
use SOAP::Lite;

sub x {
    my $self = shift;
    Transaction(Compare($self, ElementOf($self)));
}

1;
), 1, "organize imports: duplicate package.");

#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::OrganizeImports', q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( Compare ElementOf );

1;
), q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools;

1;
), 2, "organize imports: completely unused elements");

#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::OrganizeImports', q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( Compare );
use feature 'switch';
use DE_EPAGES::FactFinder::API::FactFinder;

Compare(1,2);

1;
), q(
package DE_EPAGES::Test::Abc;
use strict;

use feature 'switch';
use DE_EPAGES::Core::API::PerlTools qw ( Compare );
use DE_EPAGES::FactFinder::API::FactFinder;

Compare(1,2);

1;
), 1, "organize imports: use feature mixed-in and *probably* not unsed are not removed.");

#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::OrganizeImports', q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( Compare Compare );

Compare(1,2);

1;
), q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( Compare );

Compare(1,2);

1;
), 1, "organize imports: removed duplicate import word");

#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::OrganizeImports', q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( Compare );


use DE_EPAGES::Core::API::Test qw ( XY );

Compare(1,2, XY());

1;
), q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( Compare );
use DE_EPAGES::Core::API::Test qw ( XY );

Compare(1,2, XY());

1;
), 0, "organize imports: removed whitespace.");


#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::OrganizeImports', q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( Compare unique
    string_unique
    trim     isValid
    GenerateGUID    GenerateShortGUID
);

Compare . unique . string_unique . trim . isValid .  GenerateGUID . GenerateShortGUID
1;
), q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw (
    Compare
    GenerateGUID
    GenerateShortGUID
    isValid
    string_unique
    trim
    unique
);

Compare . unique . string_unique . trim . isValid .  GenerateGUID . GenerateShortGUID
1;
), 0, "organize imports: newlines and alphabetic sorting.");

#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::OrganizeImports', q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( Compare unique );

Compare(1,2);

package DE_EPAGES::Test::Abcdef;
use strict;

1;
), q(
package DE_EPAGES::Test::Abc;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( Compare unique );

Compare(1,2);

package DE_EPAGES::Test::Abcdef;
use strict;

1;
), 0, "organize imports: does nothing on multiple packages");

#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::OrganizeImports', q(
package DE_EPAGES::Test::Abc;

sub x {
    my $self = shift;
    Transaction(Compare($self, ElementOf($self)));
}

1;
), q(
package DE_EPAGES::Test::Abc;

use DE_EPAGES::Core::API::PerlTools qw ( Compare ElementOf );

sub x {
    my $self = shift;
    Transaction(Compare($self, ElementOf($self)));
}

1;
), 3, "organize imports: no use statements yet. Transaction is ambigous.");

#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::OrganizeImports', 
    q(use DE_EPAGES::Core::API::PerlTools qw ( Compare );

sub x {
    my $self = shift;
    Compare($self, ElementOf($self));
}

1;
), q(use DE_EPAGES::Core::API::PerlTools qw ( Compare ElementOf );

sub x {
    my $self = shift;
    Compare($self, ElementOf($self));
}

1;
), 1, "organize imports: use statement is first line.");


}


sub testCorrectStringySplit {
    my $critic = getCritic();
    $critic->add_policy( -policy => 'Perl::Critic::Policy::BuiltinFunctions::ProhibitStringySplit' );
    ##no critic qw( ProhibitImplicitNewlines )
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::CorrectStringySplit', q{
split ',', 'Alle,meine,Entchen';
split '/', 'Alle/meine/Entchen';
}, q{
split /,/, 'Alle,meine,Entchen';
split /\//, 'Alle/meine/Entchen';
}, 2, 'stringy split: simple and containing slashes');
    ##no critic qw( ProhibitImplicitNewlines )
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::CorrectStringySplit', q{
split '//', 'Alle//meine//Entchen';
}, q{
split /\/\//, 'Alle//meine//Entchen';
}, 1, 'stringy split: multiple slashes');

}


sub testCorrectDoubleSigils {
    my $critic = getCritic();
    $critic->add_policy( -policy => 'Perl::Critic::Policy::References::ProhibitDoubleSigils' );
    ##no critic qw( ProhibitImplicitNewlines )
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::CorrectDoubleSigils', q{
        @$aBlabla;
        foreach my $a (keys %$hBlabla2) {
            ++$a;
        }
}, q{
        @{$aBlabla};
        foreach my $a (keys %{$hBlabla2}) {
            ++$a;
        }
}, 2, 'double sigils: simple');
}

sub testChangeBooleanGrep {
    my $critic = getCritic();
    $critic->add_policy( -policy => 'Perl::Critic::Policy::BuiltinFunctions::ProhibitBooleanGrep' );
    ##no critic qw( ProhibitImplicitNewlines )
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::ChangeBooleanGrep', q{
package Test::Abc;
print 'match' if (grep { $_ =~ /This/ }, qw( This is a Test ));
1;
}, q{
package Test::Abc;
use List::MoreUtils qw ( any );

print 'match' if (any { $_ =~ /This/ }, qw( This is a Test ));
1;
}, 1, 'boolean grep: without existing uses');

#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::ChangeBooleanGrep', q{
package Test::Abc;
use My::Blabla qw (Test);
print 'match' if (grep { $_ =~ /This/ }, qw( This is a Test ));
1;
}, q{
package Test::Abc;
use List::MoreUtils qw ( any );
use My::Blabla qw ( Test );

print 'match' if (any { $_ =~ /This/ }, qw( This is a Test ));
1;
}, 1, 'boolean grep: with existing uses');

}

sub testAddTrailingComma {
    my $critic = getCritic();
    $critic->add_policy( -policy => 'Perl::Critic::Policy::CodeLayout::RequireTrailingCommas' );
    ##no critic qw( ProhibitImplicitNewlines )
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::AddTrailingComma', q{
    my %Params = (
        'METHOD'         => 'DoDirectPayment',
        'Test'           => 'Test',
        'Test2'          => 'Test2'
    );
},q{
    my %Params = (
        'METHOD'         => 'DoDirectPayment',
        'Test'           => 'Test',
        'Test2'          => 'Test2',
    );
}, 1, 'trailing comma: simple');
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::AddTrailingComma', q{
    my %Params = (
        'METHOD'         => 'DoDirectPayment',
        'Test'           => 'Test',
        'Test2'          => 'Test2'
    );
},q{
    my %Params = (
        'METHOD'         => 'DoDirectPayment',
        'Test'           => 'Test',
        'Test2'          => 'Test2'
    );
}, 0, 'trailing comma: location mismatch', '1,1,1');

}

sub testCorrectIndirectSyntax {
    my $critic = getCritic();
    $critic->add_policy( -policy => 'Perl::Critic::Policy::Objects::ProhibitIndirectSyntax' );
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::CorrectIndirectSyntax', q{
    my $Obj = new DateTime::Format::Builder;
},q{
    my $Obj = DateTime::Format::Builder->new;
}, 1, 'indirect syntax: simple case');
#---------------------------------------------------------------------------
    _testRule($critic,
    'DE_EPAGES::Core::API::PPI::Transform::Rule::CorrectIndirectSyntax', q{
    my $Obj = new DateTime::Format::Builder($a, $b, $c);
},q{
    my $Obj = DateTime::Format::Builder->new($a, $b, $c);
}, 1, 'indirect syntax: with arguments');

}

sub _testRule {
    my ($critic, $Rule, $sourceCode, $outputCode, $plannedViolations, $Description, $LocationOnly) = @_;

    UsePackage($Rule);
    my $Refactor = $Rule->new();
    is(
        @{$Refactor->violations([$critic->critique( \$sourceCode )], $LocationOnly)},
        $plannedViolations,
        $plannedViolations . ' planned violations for '.$Description
    ) or diag(@{$Refactor->violations()});
    my $Document = PPI::Document->new(\$sourceCode);
    $Document->index_locations;
    $Refactor->document($Document);

    cmp_ok($Document->serialize, 'eq', $outputCode, 'source code compare for  '.$Description);
}

RunScript( Sub=>\&Main );
