use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  JoinObjectPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::MeinPaket::t::TestHelper qw ( RunWithRemoteCall );
use Readonly;
use Test::More;

Readonly my $ROOTPATH => '/Test';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

                # tests
            RunWithRemoteCall(sub {
                testLogin();
                testCreateOffer(_loadProduct('cg_0101104470'));
                testUploadOffer();
                testUpdateOffer();
                testDeleteOffer();
                testDeleteProduct(_loadProduct('cg_0101104470'));
            });

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'import.xml', $ROOTPATH );

    my $Config = _loadPortalConfig();

    # initialize with our test account.
    $Config->set( {
            'PortalMerchantID'       => 'cartridge@epages.com',
            'PortalMerchantPassword' => 'MeinPaket2010',
        }
    );

    return 1;
}

sub testLogin {
    my $Config = _loadPortalConfig();
    $Config->login();
    pass('login successful.');
}

sub testCreateOffer {
    my ($Product) = @_;
    my $Config  = _loadPortalConfig();

    # did not work in import.xml... ?
    # testing wrong EAN.
    $Product->set( { 'Name' => 'Deuter Kangaroo', 'UPCEAN' => 'epages' }, GetPKeyLanguageByCode('de') );

    my $aOffers;
    my $aWarnings = BlockWarnings(
        sub {
            $aOffers = $Config->addProducts( [$Product] );
        }
    );
    is( scalar @{$aWarnings}, 0, 'dont upload invalid EAN.' ) or diag( @{$aWarnings} );
    is( scalar @{$aOffers}, 1, 'one created offer' );

    my $Offer = $aOffers->[0];
    isa_ok( $Offer, 'DE_EPAGES::MeinPaket::API::Object::MeinPaketOffer' );

    $aWarnings = BlockWarnings(
        sub {
            $Config->addProducts( [$Product] );
        }
    );
    is( scalar @{$aWarnings}, 2, 'warning on duplicate add' ) or diag( @{$aWarnings} );
    is( scalar @{ $Config->get('MeinPaketOffers') }, 1, 'one offer total' );

}

sub testUploadOffer {
    my $Config  = _loadPortalConfig();
    my $aOffers = $Config->get('MeinPaketOffers');
    my $Offer   = $aOffers->[0];
    isa_ok( $Offer, 'DE_EPAGES::MeinPaket::API::Object::MeinPaketOffer' );
    my $Category = _loadCategory('Equipment');
    $Offer->set( {
            MeinPaketCategory => '3.4.9.23',
            Category => $Category,
            ShortDescription  => 'short description',
            DeliveryLatency   => '2',
        },
        GetPKeyLanguageByCode('de')
    );
    ok( $Offer->isComplete(), 'offer settings complete' );

    my $aWarnings = BlockWarnings(
        sub {
            $Config->upload( [$Offer], undef, 'start' );
        }
    );
    is( scalar @{$aWarnings}, 0, 'no warnings on upload' ) or diag( @{$aWarnings} );
}

sub testUpdateOffer {
    my $Config  = _loadPortalConfig();
    my $aOffers = $Config->get('MeinPaketOffers');
    my $Offer   = $aOffers->[0];
    my $Product = $Offer->get('Product');

    my $aWarnings = BlockWarnings(
        sub {
            $Config->upload( [], ['Stock'] );
        }
    );
    is( scalar @{$aWarnings}, 0, 'upload stock with empty list ok' ) or diag( @{$aWarnings} );

     $aWarnings = BlockWarnings(
        sub {
            $Config->upload( $Config->get('MeinPaketOffers'), ['Stock'] );
        }
    );
    is( scalar @{$aWarnings}, 0, 'upload stock with non-empty list ok' ) or diag( @{$aWarnings} );

    $Product->set({'StockLevel' => -1 });
     $aWarnings = BlockWarnings(
        sub {
            $Config->upload( $Config->get('MeinPaketOffers'), ['Stock'] );
        }
    );
    is( scalar @{$aWarnings}, 0, 'upload negative stock ok' ) or diag( @{$aWarnings} );

    $Product->set({'StockLevel' => 10 });
}

sub testDeleteOffer {
    my $Config  = _loadPortalConfig();
    my $aOffers = $Config->get('MeinPaketOffers');
    my $Offer   = $aOffers->[0];
    isa_ok( $Offer, 'DE_EPAGES::MeinPaket::API::Object::MeinPaketOffer' );

    my $aWarnings = BlockWarnings(
        sub {
            $Config->deleteOffers( [$Offer] );
        }
    );
    is( scalar @{$aWarnings}, 0, 'no warnings on upload' ) or diag( @{$aWarnings} );
    is( scalar @{ $Config->get('MeinPaketOffers') }, 0, 'after delete: no more offers' );
}

sub testDeleteProduct {
    my ($Product) = @_;
    my $Config  = _loadPortalConfig();
    $Product->delete;
    is( scalar @{ $Config->get('MeinPaketOffers') }, 0, 'after product delete: no more offers' );
}

sub _loadCategory{
    my ($Alias) = @_;
    my $Category = LoadObjectByPath($ROOTPATH)->path( JoinObjectPath( [ 'TestShopMeinPaket', 'Categories','Equipment' ] ) );
    isa_ok( $Category, 'DE_EPAGES::Content::API::Object::Category' );
    return $Category;
}

sub _loadProduct {
    my ($Alias) = @_;
    my $Product = LoadObjectByPath($ROOTPATH)->path( JoinObjectPath( [ 'TestShopMeinPaket', 'Products', $Alias ] ) );
    isa_ok( $Product, 'DE_EPAGES::Product::API::Object::Product' );
    return $Product;
}

sub _loadPortalConfig {
    my $Config = LoadObjectByPath($ROOTPATH)->path( JoinObjectPath( [ 'TestShopMeinPaket', 'PortalConfigs', 'MeinPaketTestConfig' ] ) );
    isa_ok( $Config, 'DE_EPAGES::MeinPaket::API::Object::MeinPaketConfig' );
    return $Config;
}

RunScript( Sub => \&Main );

1;
