use Test::More tests => 309;
use strict;
use Readonly;
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime );
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo GetCustomerFolder );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath ExistsObjectByPath );
use DE_EPAGES::Order::API::Shop qw ( GetNextOrderNo );
use DE_EPAGES::Order::API::OrderDocument qw ( CreateOrderDocument );
use Time::HiRes qw( gettimeofday tv_interval );

Readonly::Scalar my $CUSTOMERNO_SIZE => 4;
Readonly::Scalar my $TESTITERATIONS => 100;
Readonly::Scalar my $MAXVALUE => 5;
Readonly::Scalar my $MAXSECONDS => 8.5; # Baselines: 6.13.1: 8.5.

my $ROOTPATH = '/Test';

sub Main {
    Init();

    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShop1');
    ok( $Shop->id, 'create shop ' . $Shop->id );

    my $CustomerNo = GetNextCustomerNo( $Shop );
    ok( length $CustomerNo >= $CUSTOMERNO_SIZE, 'customer no. starts with 1000');

    my $CustomerFolder = GetCustomerFolder( $Shop );
    ok( !$CustomerFolder->existsChild( $CustomerNo ), 'customer no. does not exists yet');

    my $Customer = $Shop->get('CustomerClass')->insertObject({
        Parent => $CustomerFolder,
        Alias => $CustomerNo,
    });
    ok( $Customer->id, 'insert new customer' );
    ok( $Customer->alias eq $CustomerNo, 'Customer.Alias' );

    # allow features for order documents
    $Shop->featureMaxValue( 'OrderDocument', $MAXVALUE );

    my $OrderNo;
    my $Order;
    my $Container;
    my @Orders;

    note "creating $TESTITERATIONS orders";
    for my $i ( 0 .. $TESTITERATIONS ) {
        $OrderNo= GetNextOrderNo( $Shop );
        $Order= $Customer->folder('Orders')->insertChild( 'CustomerOrder', {
            'Alias'     => $OrderNo,
            'Shop'      => $Shop,
            'Customer'  => $Customer,
        } );
        ok( $Order->id, 'insert new order' );
        ok( $Order->alias eq $OrderNo, 'Order.Alias' );
        ok( !defined $Order->get('LastStatus'), 'new order has no status' );

        # add a line item container
        $Container = $Order->insertChild('LineItemContainer', {
            'Alias'             => 'LineItemContainer',
            'LanguageID'        => $Shop->get('LanguageID'),
            'LocaleID'          => $Shop->get('LocaleID'),
            'CurrencyID'        => $Shop->get('CurrencyID'),
            'TaxArea'           => $Shop->get('TaxMatrix')->defaultTaxArea,
            'TaxModel'          => $Shop->get('TaxModel'),
        });
        $Order->set({'LineItemContainer' => $Container});
        push(@Orders, $Order);
    }

    # Creation date from form
    my $DeliveryDate = GetCurrentDateTime();
    $DeliveryDate->truncate( 'to' => 'day' );
    my $IssueDate = $DeliveryDate;

    # Create an invoice and packing slip for each order.
    note "creating order documents";
    my $t0= [gettimeofday];
DB::enable_profile() if DB->can('enable_profile');
    foreach $Order (@Orders) {
        CreateOrderDocument ( $Order, 'Invoice', 'Invoices', {
            'DeliveryDate' => $DeliveryDate,
            'IssueDate' => $IssueDate
        });

        CreateOrderDocument ( $Order, 'PackingSlip', 'PackingSlips', {
          'DeliveryDate' => $DeliveryDate,
          'IssueDate' => $IssueDate
        });
    }
DB::disable_profile() if DB->can('disable_profile');

    my $t1= [gettimeofday];
    my $elapsed = tv_interval ($t0, $t1);

    cmp_ok( $elapsed, '<', $MAXSECONDS, sprintf("Creating $TESTITERATIONS invoices and packing slips takes less "
                                              . "than $MAXSECONDS seconds. This run took: %0.2f seconds.", $elapsed) );

    Cleanup();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    $Folder->insertChild( 'Shop', { Alias => 'TestShop1' } );
    return 1;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
