use strict;
use Test::More;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath);

#========================================================================================
# function     TimeThis
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Time = TimeThis( $Code );
#----------------------------------------------------------------------------------------
# description  Executes a code block and returns the execution time.
#----------------------------------------------------------------------------------------
# input        $Code | code to evaluate | code reference
# return       $Time | execution time in seconds | float
#========================================================================================
sub TimeThis {
    my ($Code) = @_;

    my $time0 = time();
    $Code->();
    return time() - $time0;
}

sub Main {

    ## no critic qw( ProhibitMagicNumbers )

    my $NumTests = 60;
    plan tests => $NumTests;

    my $ObjectPath = '/Shops/DemoShop';
    my $DemoShopURL;
    my $ProductID;
    my $ProductURL;
    my $IsPageCacheUsed;
    my $LocaleID;
    my $OrderProcess;
    RunOnStore( Store => GetTestSite(), Sub => sub {
        if( ExistsObjectByPath( $ObjectPath ) ) {
            my $Shop = LoadObjectByPath( $ObjectPath );
            $DemoShopURL = $Shop->get('WebUrl');
            $IsPageCacheUsed = $Shop->get('IsPageCacheUsed');
            $LocaleID = $Shop->get('LocaleID');

            my $Product = $Shop->get('ProductFolder')->child('ho_40407')->child('SubProducts')->child('ho_4040705005');
            $ProductID = $Product->id;
            $ProductURL = $Product->get('WebUrl');
            $OrderProcess = $Shop->get('OrderProcess');

            $Shop->set({ 'OrderProcess' => 1 }); # in case a previous run failed
        }
    });

    SKIP: {
        skip 'no DemoShop installed', $NumTests unless defined $DemoShopURL;

        ok( !$IsPageCacheUsed, 'Page Cache is disabled' );
        is( $LocaleID, 'en_GB', 'default Locale is en_GB' );

        my $Test = MY::Test->new(
            DemoShopURL => $DemoShopURL,
            ProductURL => $ProductURL,
            ProductID => $ProductID,
        );

        my $PreloadNewProcessTime = TimeThis( sub { $Test->testPreloadNew } );
        cmp_ok( $PreloadNewProcessTime, '<', 20, sprintf('preload new: %0.2f s', $PreloadNewProcessTime) );

        my $NewProcessTime = TimeThis( sub { $Test->testNewOrderProcess } );
        cmp_ok( $NewProcessTime, '<', 10, sprintf('new order process: %0.2f s', $NewProcessTime) );

        # set the old order process
        RunOnStore( Store => GetTestSite(), Sub => sub {
            LoadObjectByPath( $ObjectPath )->set({ 'OrderProcess' => 0 });
        });

        my $PreloadClassicProcessTime = TimeThis( sub { $Test->testPreloadClassic   } );
        cmp_ok( $PreloadClassicProcessTime, '<', 10, sprintf('preload classic: %0.2f s', $PreloadClassicProcessTime) );

        my $ClassicProcessTime = TimeThis( sub { $Test->testClassicOrderProcess } );
        cmp_ok( $ClassicProcessTime, '<', 10, sprintf('classic order process: %0.2f s', $ClassicProcessTime) );

        # set the previous order process
        RunOnStore( Store => GetTestSite(), Sub => sub {
            LoadObjectByPath( $ObjectPath )->set({ 'OrderProcess' => $OrderProcess });
        });
    };

    return;
}


RunScript( Sub => \&Main, Debug => 1 );




package MY::Test; ## no critic qw(RequireCommentForExportedFunctions);

use strict;
use Test::More; ## no critic qw(RequireCorrectImports)
use Time::HiRes qw (time); ## no critic qw(RequireCorrectImports)
use DE_EPAGES::Core::API::File qw( DeleteFile WriteFile );
use DE_EPAGES::Core::API::Unicode qw( Unicode2Filename );
use DE_EPAGES::Core::API::Url qw( BuildParameterString );
use DE_EPAGES::WebInterface::API::FakeRequest;

sub new {
    my $class = shift;
    my %options = @_;

    return bless {
        %options,
        FakeRequest => DE_EPAGES::WebInterface::API::FakeRequest->new,
    }, $class;
}

sub fakeRequest {
    return shift->{FakeRequest};
}

sub demoShopURL {
    return shift->{DemoShopURL};
}

sub productURL {
    return shift->{ProductURL};
}

sub productID {
    return shift->{ProductID};
}

sub addToBasketURL {
    my $self = shift;
    return $self->productURL . '&ChangeAction=AddToBasket&Quantity=2&ViewObjectID=' . $self->productID;
}

sub get {
    my $self = shift;
    my ($URL) = @_;

    my $Response = $self->fakeRequest->get( $URL );
    if( !$Response->is_success ) {
        die "Can't load '$URL': " . $Response->status_line;
    }
    else {
        return $Response->content;
    }
}

sub post {
    my $self = shift;
    my ($URL, $hParams) = @_;

    my $Response = $self->fakeRequest->post( $URL, BuildParameterString($hParams) );
    if( !$Response->is_success ) {
        die "Can't load '$URL': " . $Response->status_line;
    }
    else {
        return $Response->content;
    }
}

sub getTitle {
    my $self = shift;
    my ($URL) = @_;

    my $Content = $self->get( $URL );
    if( $Content =~ /<title.*?>(.*?)<\/title>/x ) {
        return $1;
    }
    else {
        die "No title found in '$URL'";
    }
}

## no critic qw(ProhibitMagicNumbers)
sub testPreloadNew { # using default order process
    my $self = shift;

    is( $self->getTitle( $self->demoShopURL ), 'Milestones - Equipped to meet your goals', 'preload new: start page' );
    is( $self->getTitle( $self->productURL ), 'Jack Wolfskin Blizzard Jacket - Milestones', 'preload new: product page' );

    my $AddToBasketContent = $self->get( $self->addToBasketURL );
    _Like( $AddToBasketContent, qr!Subtotal!, 'preload new: add to basket' );

    my $BasketID = _BasketID( $AddToBasketContent );
    my $BaseURL = _BaseURL( $AddToBasketContent );

    my $BasketURL = "$BaseURL?ObjectID=$BasketID";
    my $BasketContent = $self->get($BasketURL );
    _Like( $BasketContent, qr/Do you wish to send this order as a gift\?/, 'preload new: basket page' );

    my $LoginURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=SaveLoginForm&Login=mmustermann&Password=123456';
    my $LoginContent = $self->get( $LoginURL );
    _Like( $LoginContent, qr/Max Mustermann/, 'preload new: login' );

    $BaseURL = _BaseURL($LoginContent);
    my $AddressURL = $BaseURL.'?ObjectID='.$BasketID.'&ViewAction=ViewMultiCheckoutAddress';
    my $AddressContent = $self->get( $AddressURL );
    _Like($AddressContent , qr/Enter a separate delivery address/, 'preload new: address selection' );

    $AddressContent =~ /name="BillingAddress" value="(\d+)"/ or die 'BillingAddress id not found';
    my $BillingAddressID = $1;

    $AddressContent =~ /name="ShippingAddress" value="(\d+)"/ or die 'ShippingAddress id not found';
    my $ShippingAddressID = $1;

    my $Content = $self->post( $BaseURL, {
        ObjectID => $BasketID,
        ViewAction => 'ViewMultiCheckoutShipping',
        ChangeAction => 'SaveMultiCheckoutAddress',
        BillingAddress => $BillingAddressID,
        ShippingAddress => $ShippingAddressID,
    });
    _Like( $Content, qr/To selection of payment type/, 'preload new: shipping selection');

    my $PaymentURL = "$BaseURL?ObjectID=$BasketID&ViewAction=ViewMultiCheckoutPayment";
    _Like( $self->get( $PaymentURL ), qr/To checking of order/, 'preload new: payment selection');

    my $ConfirmURL = "$BaseURL?ObjectID=$BasketID&ViewAction=ViewMultiCheckoutConfirmation";
    _Like( $self->get( $ConfirmURL ), qr/Place order/, 'preload new: confirmation page' );

    my $OrderURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=TestTermsAndConditions&tac=1&ChangeAction=SaveMultiCheckoutConfirmation';
    _Like( $self->get( $OrderURL ), qr/Thank you\. We have received your order/, 'preload new: order' );
    return;
}

sub testNewOrderProcess {
    my $self = shift;

#DB::enable_profile() if DB->can('enable_profile');
    my $StartPageTime = TimeThis( sub {
        is( $self->getTitle( $self->demoShopURL ), 'Milestones - Equipped to meet your goals', 'start page' );
    } );
    cmp_ok( $StartPageTime, '<', 0.6, sprintf('new process: start page: %0.2f s', $StartPageTime) );
#DB::disable_profile() if DB->can('disable_profile');;

    my $ProductPageTime = TimeThis( sub {
        is( $self->getTitle( $self->productURL ), 'Jack Wolfskin Blizzard Jacket - Milestones', 'preload product page' );
    } );
    cmp_ok( $ProductPageTime, '<', 0.9, sprintf('new process: product page: %0.2f s', $ProductPageTime) );

    # add to basket
    my $BaseURL;
    my $BasketID;
    my $AddToBasketTime = TimeThis( sub {
        my $AddToBasketContent = $self->get( $self->addToBasketURL );
        _Like( $AddToBasketContent, qr!Subtotal!, 'new process: add to basket' );

        $BasketID = _BasketID( $AddToBasketContent );
        $BaseURL = _BaseURL( $AddToBasketContent );
    } );
    cmp_ok( $AddToBasketTime, '<', 2, sprintf('new process: add to basket: %0.2f s', $AddToBasketTime) );

    my $BasketURL = $BaseURL.'?ObjectID='.$BasketID;
    my $BasketTime = TimeThis( sub {
        my $BasketContent = $self->get($BasketURL );
        _Like( $BasketContent, qr/Do you wish to send this order as a gift\?/, 'new process: basket page' );
    });
    cmp_ok( $BasketTime, '<', 1, sprintf('new process: basket: %0.2f s', $BasketTime) );

    my $LoginTime = TimeThis( sub {
        my $LoginURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=SaveLoginForm&Login=mmustermann&Password=123456';
        my $LoginContent = $self->get( $LoginURL );
        _Like( $LoginContent, qr/Max Mustermann/, 'new process: login' );

        $BaseURL = _BaseURL($LoginContent);
    });
    cmp_ok( $LoginTime, '<', 1.5, sprintf('new process: login: %0.2f s', $LoginTime) );

    my ($BillingAddressID, $ShippingAddressID);
    my $AddressTime = TimeThis( sub {
        my $AddressURL = $BaseURL.'?ObjectID='.$BasketID.'&ViewAction=ViewMultiCheckoutAddress';
        my $AddressContent = $self->get( $AddressURL );
        _Like($AddressContent , qr/Enter a separate delivery address/, 'new process: address selection' );

        $AddressContent =~ /name="BillingAddress" value="(\d+)"/ or die 'BillingAddress id not found';
        $BillingAddressID = $1;

        $AddressContent =~ /name="ShippingAddress" value="(\d+)"/ or die 'ShippingAddress id not found';
        $ShippingAddressID = $1;
    });
    cmp_ok( $AddressTime, '<', 1.2, sprintf('new process: address: %0.2f s', $AddressTime) );

    my $ShippingTime = TimeThis( sub {
        my $Content = $self->post( $BaseURL, {
            ObjectID => $BasketID,
            ViewAction => 'ViewMultiCheckoutShipping',
            ChangeAction => 'SaveMultiCheckoutAddress',
            BillingAddress => $BillingAddressID,
            ShippingAddress => $ShippingAddressID,
        });
        _Like( $Content, qr/To selection of payment type/, 'new process: shipping selection');
    });
    cmp_ok( $ShippingTime, '<', 0.8, sprintf('new process: shipping: %0.2f s', $ShippingTime) );

    my $PaymentTime = TimeThis( sub {
        my $ConfirmURL = "$BaseURL?ObjectID=$BasketID&ViewAction=ViewMultiCheckoutPayment";
        _Like( $self->get( $ConfirmURL ), qr/To checking of order/, 'new process: payment selection');
    });
    cmp_ok( $PaymentTime, '<', 0.8, sprintf('new process: payment: %0.2f s', $PaymentTime) );

    my $ConfirmTime = TimeThis( sub {
        my $ConfirmURL = "$BaseURL?ObjectID=$BasketID&ViewAction=ViewMultiCheckoutConfirmation";
        _Like( $self->get( $ConfirmURL ), qr/Place order/, 'new process: confirmation page');
    });
    cmp_ok( $ConfirmTime, '<', 1, sprintf('new process: confirm: %0.2f s', $ConfirmTime) );

    my $OrderTime = TimeThis( sub {
        my $OrderURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=TestTermsAndConditions&tac=1&ChangeAction=SaveMultiCheckoutConfirmation';
        _Like( $self->get( $OrderURL ), qr/Thank you\. We have received your order/, 'new process: order' );
    });
    cmp_ok( $OrderTime, '<', 1.5, sprintf('new process: order: %0.2f s', $OrderTime) );
}

sub testPreloadClassic {
    my $self = shift;

    is( $self->getTitle( $self->demoShopURL ), 'Milestones - Equipped to meet your goals', 'preload classic: start page' );
    is( $self->getTitle( $self->productURL ), 'Jack Wolfskin Blizzard Jacket - Milestones', 'preload classic: product page' );

    # add to basket
    my $AddToBasketContent = $self->get( $self->addToBasketURL );
    _Like( $AddToBasketContent, qr!Total amount!, 'preload classic: add to basket' );

    my $BasketID = _BasketID( $AddToBasketContent );
    my $BaseURL = _BaseURL( $AddToBasketContent );
    my $BasketURL = $BaseURL.'?ObjectID='.$BasketID;
    my $BasketContent = $self->get($BasketURL );
    _Like( $BasketContent, qr/Order and delivery comments/, 'preload classic: basket page' );

    my $LoginURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=SaveLoginForm&Login=mmustermann&Password=123456';
    my $LoginContent = $self->get( $LoginURL );
    _Like( $LoginContent, qr/Max Mustermann/, 'preload classic: login' );

    $BaseURL = _BaseURL($LoginContent);

    my $BillingAddressID;
    my $AddressURL = $BaseURL.'?ObjectID='.$BasketID.'&ViewAction=ViewAddress';
    my $AddressContent = $self->get( $AddressURL );
    _Like($AddressContent , qr/Add a new delivery address/, 'preload classic: address selection' );

    $AddressContent =~ /name="BillingAddress" value="(\d+)"/ or die 'BillingAddress id not found';
    $BillingAddressID = $1;

    my $ConfirmURL = "$BaseURL?ObjectID=$BasketID&ChangeAction=SelectAddress&ChangeAction=DoOrder&BillingAddress=$BillingAddressID&ShippingAddress=$BillingAddressID";
    _Like( $self->get( $ConfirmURL ), qr/Place order/, 'preload classic: confirmation page' );

    my $OrderURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=TestTermsAndConditions&tac=1&ChangeAction=DoOrder';
    _Like( $self->get( $OrderURL ), qr/Confirmation/, 'preload classic: order' );
}

sub testClassicOrderProcess {
    my $self = shift;

#DB::enable_profile() if DB->can('enable_profile');
    my $StartPageTime = TimeThis( sub {
        is( $self->getTitle( $self->demoShopURL ), 'Milestones - Equipped to meet your goals', 'classic process: start page' );
    } );
    cmp_ok( $StartPageTime, '<', 0.6, sprintf('classic process:start page: %0.2f s', $StartPageTime) );
#DB::disable_profile() if DB->can('disable_profile');;

    my $ProductPageTime = TimeThis( sub {
        is( $self->getTitle( $self->productURL ), 'Jack Wolfskin Blizzard Jacket - Milestones', 'classic process: product page' );
    } );
    cmp_ok( $ProductPageTime, '<', 0.9, sprintf('classic process:product page: %0.2f s', $ProductPageTime) );

    # add to basket
    my $BaseURL;
    my $BasketID;
    my $AddToBasketTime = TimeThis( sub {
        my $AddToBasketContent = $self->get( $self->addToBasketURL );
        _Like( $AddToBasketContent, qr!Total amount!, 'classic process: add to basket' );

        $BasketID = _BasketID( $AddToBasketContent );
        $BaseURL = _BaseURL( $AddToBasketContent );
    } );
    cmp_ok( $AddToBasketTime, '<', 2, sprintf('classic process: add to basket: %0.2f s', $AddToBasketTime) );

    my $BasketURL = $BaseURL.'?ObjectID='.$BasketID;
    my $BasketTime = TimeThis( sub {
        my $BasketContent = $self->get($BasketURL );
        _Like( $BasketContent, qr/Order and delivery comments/, 'classic process: basket page' );
    });
    cmp_ok( $BasketTime, '<', 1.0, sprintf('classic process: basket: %0.2f s', $BasketTime) );

    my $LoginTime = TimeThis( sub {
        my $LoginURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=SaveLoginForm&Login=mmustermann&Password=123456';
        my $LoginContent = $self->get( $LoginURL );
        _Like( $LoginContent, qr/Max Mustermann/, 'classic process: login' );

        $BaseURL = _BaseURL($LoginContent);
    });
    cmp_ok( $LoginTime, '<', 1.5, sprintf('classic process:login: %0.2f s', $LoginTime) );

    my $BillingAddressID;
    my $AddressTime = TimeThis( sub {
        my $AddressURL = $BaseURL.'?ObjectID='.$BasketID.'&ViewAction=ViewAddress';
        my $AddressContent = $self->get( $AddressURL );
        _Like($AddressContent , qr/Add a new delivery address/, 'classic process: address selection' );

        $AddressContent =~ /name="BillingAddress" value="(\d+)"/ or die 'BillingAddress id not found';
        $BillingAddressID = $1;
    });
    cmp_ok( $AddressTime, '<', 0.8, sprintf('classic process:address: %0.2f s', $AddressTime) );

    my $ConfirmTime = TimeThis( sub {
        my $ConfirmURL = "$BaseURL?ObjectID=$BasketID&ChangeAction=SelectAddress&ChangeAction=DoOrder&BillingAddress=$BillingAddressID&ShippingAddress=$BillingAddressID";
        _Like( $self->get( $ConfirmURL ), qr/Place order/, 'classic process: confirmation page' );
    });
    cmp_ok( $ConfirmTime, '<', 1, sprintf('classic process: confirm: %0.2f s', $ConfirmTime) );

    my $OrderTime = TimeThis( sub {
        my $OrderURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=TestTermsAndConditions&tac=1&ChangeAction=DoOrder';
        _Like( $self->get( $OrderURL ), qr/Confirmation/, 'classic process: order' );
    });
    cmp_ok( $OrderTime, '<', 1.5, sprintf('classic process: order: %0.2f s', $OrderTime) );
}

sub _Like($$$) {
    my ($Content, $RegExp, $Message) = @_;

    my $FileName = Unicode2Filename($Message).'.html';
    DeleteFile( $FileName ); # in case it was created by a previous run

    my $ok = $Content =~ $RegExp;
    ok( $ok, $Message );

    unless ($ok) {
        WriteFile( $FileName, \$Content );
        diag "Content does not match $RegExp";
        note "Content was saved as $FileName";
    }
}

sub _BasketID {
    my ($Content) = @_;

    if( $Content =~ m!
                        <a\s+
                          class="MiniCheckoutLink\s+.*?Basket.*?"\s+
                          href="\?ObjectID=(\d+)"\s*
                        >
                     !x ) { ## no critic qw(ProhibitComplexRegexes)
        return $1;
    }
    die 'Basket link not found';
}

sub _BaseURL {
    my ($Content) = @_;

    if( $Content =~ m!<base\s+href="(.*?)"!x ) {
        return $1;
    }
print $Content;
    die 'base tag not found';
}

sub TimeThis {
    my ($Code) = @_;

    my $time0 = time();
    $Code->();
    return time() - $time0;
}

