use Test::More tests => 36;

use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( GetRootObjectID );

RunScript( Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main )
});

sub Main {
    # test defaults
    my $Pager = MyPager->new({
        ObjectID => GetRootObjectID(),
        _itemCount_ => 34,
    });
    is( $Pager->get('Page'), 1, 'default page number' );
    is( $Pager->get('PageSize'), 10, 'default page size' );
    is( $Pager->get('PagerSize'), 10, 'default pager size' );
    is( $Pager->get('OrderBy'), '', 'default sort column' );
    is( $Pager->get('OrderDesc'), 0, 'default sort order' );
    is_deeply( $Pager->get('UriParameters'), {}, 'default UriParameters' );

    $Pager->statistic;
    is( $Pager->countItems, 34, 'countItems' );
    is( scalar @{ $Pager->allItems }, 34, 'allItems' );
    is( scalar @{ $Pager->items(1) }, 10, 'items(1)' );
    is( scalar @{ $Pager->items(4) }, 4, 'items(4)' );
    is( scalar @{ $Pager->items(5) }, 0, 'items(5)' );
    #is( scalar @{ $Pager->items(0) }, 0, 'items(0)' );

    # test custom settings
    $Pager = MyPager->new({
        ObjectID => GetRootObjectID(),
        Page => 5,
        PageSize => 4,
        PagerSize => 6,
        OrderBy => 'X',
        OrderDesc => 1,
        _itemCount_ => 34,
    });
    is( $Pager->get('Page'), 5, 'attribute Page' );
    is( $Pager->get('PageSize'), 4, 'attribute PageSize' );
    is( $Pager->get('PagerSize'), 6, 'attribute PagerSize' );
    is( $Pager->get('OrderBy'), 'X', 'attribute OrderBy' );
    is( $Pager->get('OrderDesc'), 1, 'attribute OrderDesc' );
    is_deeply( $Pager->get('UriParameters'), {}, 'default UriParameters' );

    $Pager->statistic;
    is( $Pager->get('CountOfItems'), 34, 'CountOfItems' );
    is( $Pager->get('CountOfPages'), 9, 'CountOfPages' );
    is( scalar @{ $Pager->items(9) }, 2, 'items(9)' );

    is_deeply( $Pager->get('UriParameters'), {}, 'default UriParameters' );

    my $hVars = $Pager->page->{'Pager'};
    is( $hVars->{'FirstPage'}, 1, 'FirstPage' );
    is( $hVars->{'LastPage'}, 9, 'LastPage' );
    is( $hVars->{'Page'}, 5, 'Page' );
    is( $hVars->{'NextPage'}, 6, 'NextPage' );
    is( $hVars->{'PreviousPage'}, 4, 'PreviousPage' );
    is_deeply( $hVars->{'Items'}, $Pager->items(5), 'Items' );

    my $PagesEx = [ map { _Button($_) } ( 2..7 ) ];
    is_deeply( $hVars->{'Pages'}, $PagesEx, 'Pages' );

    my $PagesNewEx = [
        _ButtonNew(1, 1),
        _ButtonNew(3, '...'),
        (map { _ButtonNew($_, $_) } ( 4..6 )),
        _ButtonNew(7, '...'),
        _ButtonNew(9, 9),
    ];
    is_deeply( $hVars->{'PagesNew'}, $PagesNewEx, 'PagesNew' );
    is( $hVars->{'URL'}, '?ViewAction=View&ObjectID=1', 'URL' );

    # edge cases for new pager: 6 pages of items
    $Pager = MyPager->new({
        ObjectID => GetRootObjectID(),
        Page => 5,
        PageSize => 6,
        _itemCount_ => 34,
    });
    $Pager->statistic;
    is( $Pager->get('CountOfPages'), 6, '6 pages' );

    $hVars = $Pager->page->{'Pager'};
    $PagesNewEx = [ map { _ButtonNew($_, $_) } ( 1..6 ) ];
    is_deeply( $hVars->{'PagesNew'}, $PagesNewEx, 'PagesNew' );

    # edge cases for new pager: 7 pages of items
    $Pager = MyPager->new({
        ObjectID => GetRootObjectID(),
        Page => 5,
        PageSize => 5,
        _itemCount_ => 34,
    });
    $Pager->statistic;
    is( $Pager->get('CountOfPages'), 7, '7 pages' );

    $hVars = $Pager->page->{'Pager'};
    $PagesNewEx = [ map { _ButtonNew($_, $_) } ( 1..7 ) ];
    is_deeply( $hVars->{'PagesNew'}, $PagesNewEx, 'PagesNew' );

    # edge cases for new pager: 8 pages of items
    $Pager = MyPager->new({
        ObjectID => GetRootObjectID(),
        Page => 5,
        PageSize => 5,
        _itemCount_ => 40,
    });
    $Pager->statistic;
    is( $Pager->get('CountOfPages'), 8, '8 pages' );

    $hVars = $Pager->page->{'Pager'};
    $PagesNewEx = [
        _ButtonNew(1, 1),
        _ButtonNew(3, '...'),
        (map { _ButtonNew($_, $_) } ( 4..8 )),
    ];
    is_deeply( $hVars->{'PagesNew'}, $PagesNewEx, 'PagesNew' );
}

sub _Button {
    my ($Page) = @_;

    return {
       'URLPage' => '?ViewAction=View&ObjectID=1' . ($Page == 1 ? '' : "&Page=$Page"),
       'Page' => $Page,
     },
}

sub _ButtonNew {
    my ($Page, $Text) = @_;

    my $hButton = _Button($Page);
    $hButton->{'PageText'} = $Text;
    return $hButton;
}


package MyPager;
use base 'DE_EPAGES::Presentation::API::Object::Pager';

use strict;

sub new {
    my $class = shift;
    my ($hParams) = @_;

    my $ItemCount = delete $hParams->{_itemCount_};
    my $self = $class->SUPER::new( $hParams );
    $self->{_itemCount_} = $ItemCount;

    return $self;
}

sub _items {
    my $self = shift;

    return [ 1..$self->{_itemCount_} ];
}

sub countItems {
    my $self = shift;

    return scalar @{ $self->_items };
}

sub items {
    my $self = shift;
    my ($Page) = @_;

    my $FirstIndex = $self->firstItemOfPage($Page);
    my $LastIndex = $self->lastItemOfPage($Page);
    return [ @{ $self->_items }[ $FirstIndex .. $LastIndex ] ];
}
