use strict;
use Test::More;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath);
use DE_EPAGES::WebInterface::API::FakeRequest;

#========================================================================================
# function     TimeThis
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Time = TimeThis( $Code );
#----------------------------------------------------------------------------------------
# description  Stops the execution time of a code block.
#----------------------------------------------------------------------------------------
# input        $Code | code to evaluate | code reference
# return       $Time | execution time in seconds | float
#========================================================================================
sub TimeThis {
    my ($Code) = @_;

    my $time0 = time();
    $Code->();
    return time() - $time0;
}

#========================================================================================
# function     Main
# state        private
#========================================================================================
sub Main {

    my $NumTests = 18;
    plan tests => $NumTests;

    my $ObjectPath = "/Shops/DemoShop";
    my $DemoShopURL;
    my $ProductURL;
    my $IsPageCacheUsed;
    my $LocaleID;
    RunOnStore(
        Store      => GetTestSite(),
        Sub        => sub {
            if( ExistsObjectByPath( $ObjectPath ) ) {
                my $Shop = LoadObjectByPath( $ObjectPath );
                $DemoShopURL = $Shop->get('WebUrl');
                $IsPageCacheUsed = $Shop->get('IsPageCacheUsed');
                $LocaleID = $Shop->get('LocaleID');

                $Shop->clearPageCache;

                my $Product = $Shop->get('ProductFolder')->child('ho_40407')->child('SubProducts')->child('ho_4040705005');
                $ProductURL = $Product->get('WebUrl');
            }
        }
    );

    SKIP: {
        skip "no DemoShop installed", $NumTests unless defined $DemoShopURL;

        my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
        my $GetURL = sub {
            my ($URL) = @_;

            my $Response = $FakeRequest->get( $URL );
            if( !$Response->is_success ) {
                die $Response->status_line;
            }
            else {
                return $Response->content;
            }
        };

        ok( !$IsPageCacheUsed, "Page Cache is disabled" );
        is( $LocaleID, 'en_GB', "default Locale is en_GB" );

        my $StartPageTime = TimeThis( sub { $GetURL->( $DemoShopURL ) } );
        cmp_ok( $StartPageTime, '<', 5, sprintf("start page: first call: %0.2f s", $StartPageTime) );

#DB::enable_profile();

        my $Sub = sub {
            my $Content = $GetURL->( $ProductURL );
            my $Title = $Content =~ /<title>(.*?)<\/title>/ ? $1 : 'no title';
            cmp_ok(  $Title, 'eq', 'Jack Wolfskin Blizzard Jacket - Milestones', "product page title" );
        };

        my $FirstTime = TimeThis( $Sub );
        cmp_ok( $FirstTime, '<', 2, sprintf("product page: first call: %0.2f s", $FirstTime) );

        my $SecondTime = TimeThis( $Sub );
        cmp_ok( $SecondTime, '<', 1.1, sprintf("product page: second call: %0.2f s", $SecondTime) );

        my $ThirdTime = TimeThis( $Sub );
        cmp_ok( $ThirdTime, '<', 1.1, sprintf("product page: third call: %0.2f s", $ThirdTime) );

#DB::disable_profile();

        # enable page cache
        RunOnStore(
            Store      => GetTestSite(),
            Sub        => sub {
                LoadObjectByPath($ObjectPath)->set({'IsPageCacheUsed' => 1 });
            }
        );

        my $ForthTime = TimeThis( $Sub );
        cmp_ok( $ForthTime, '<', 1.1, sprintf("product page: build page cache: %0.3f s", $ForthTime) );
        cmp_ok( $ForthTime, '>', 0.3, sprintf("product page: build page cache: %0.3f s", $ForthTime) );

        my $FifthTime = TimeThis( $Sub );
        cmp_ok( $FifthTime, '<', 0.072, sprintf("product page: use page cache: %0.3f s", $FifthTime) );

        # disable page cache
        RunOnStore(
            Store      => GetTestSite(),
            Sub        => sub {
                LoadObjectByPath($ObjectPath)->set({'IsPageCacheUsed' => 0 });
            }
        );

        my $SixtTime = TimeThis( $Sub );
        cmp_ok( $SixtTime, '<', 1.3, sprintf("product page: page cache disabled: %0.3f s", $SixtTime) );

        # add to basket
        my $AddToBasketURL = $ProductURL . '&ChangeAction=AddToBasket&Quantity=2';
        my $AddToBasketTime = TimeThis( sub { $GetURL->( $AddToBasketURL ) } );
        cmp_ok( $AddToBasketTime, '<', 6, sprintf("add to basket: %0.2f s", $AddToBasketTime) );

        my $AddToBasketTime2 = TimeThis( sub { $GetURL->( $AddToBasketURL ) } );
        cmp_ok( $AddToBasketTime2, '<', 3, sprintf("add to basket: %0.2f s", $AddToBasketTime2) );
    };

    return;
}

RunScript( Sub => \&Main, Debug => 1 );
