use strict;
use Test::More tests => 20;
use DE_EPAGES::SEO::API::SEO qw (GetSEOCheckValues CalculateSEOQuality GetWordCount);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

## no critic qw( ProhibitLongLines )
sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $Test = LoadObjectByPath('Test');
    my $Shop = $Test->insertChild('Shop', {
        Alias => 'TestShop',
        InitialShopWizardStartup => 0
    });
    my $ShopLangID = $Shop->get('LanguageID');

    $Shop->featureMaxValue( LoadObjectByPath('/Features/Pages'), 10 ); ## no critic qw(ProhibitMagicNumbers)

    # start page seo test
    my $hValues = GetSEOCheckValues($Shop, $Shop->get('Categories'), $ShopLangID);
    # in clean shop SEO should be very bad:
    ok(!$hValues->{'Name'}, 'GetSEOCheckValues empty name');
    cmp_ok($hValues->{'Url'}->{'Value'}, 'eq', $Shop->get('Categories')->get('CanonicalURL', $ShopLangID), 'GetSEOCheckValues default URL');
    ok(!$hValues->{'Title'}->{'Value'}, 'GetSEOCheckValues empty title');
    ok(!$hValues->{'Title'}->{'Value'}, 'GetSEOCheckValues empty title');
    ok(!$hValues->{'Description'}->{'Value'}, 'GetSEOCheckValues empty desc');
    ok(!$hValues->{'AdvDescription'}->{'Value'}, 'GetSEOCheckValues empty long desc');
    foreach my $Token ( qw (Url Title Description AdvDescription )) {
        cmp_ok($hValues->{$Token}->{'RatingClass'}, 'eq', 'red', 'GetSEOCheckValues RatingClass red');
    }
    cmp_ok(CalculateSEOQuality($hValues, $Shop->get('Categories')), '==', 0, 'CalculateSEOQuality returns 0');

    # use article and set some SEO-nice values
    $Shop->set({ 'DomainName' => 'www.mytestshop.com' });
    $Shop->set({ 'Name' => 'MyTestShop' }, $ShopLangID);
    my $Art1 = $Shop->get('Categories')->insertChild('Article', { Alias => 'Article' });
    $Art1->set({
        'Name' => 'Nice test article - all u ever wanted to know!',
        'Description' => 'Curabitur pretium elementum purus, ac tincidunt tellus lobortis id. In quis blandit purus. Quisque quis elit varius est faucibus.',
        'Text' => 'Nunc arcu <em>metus</em>, dapibus in bibendum et, aliquet eu lectus. Donec ipsum tellus, tempor non dignissim at, vehicula lacinia enim. Morbi dignissim lacus eu ipsum placerat sit amet volutpat leo vestibulum.',
        'URI' => 'top-article-kat1',
    }, $ShopLangID);

    # general attributes
    my $hGeneralValues = {
        FormerSEOQuality => $Art1->get('SEOQuality') || 0,
        SiteUrl => $Shop->get('WebUrl'),
        Domain => $Shop->get('DomainName'),
    };
    TriggerHook('SEOCheckTests', {
        'Object'        => $Art1,
        'Shop'          => $Shop,
        'Values'        => $hGeneralValues,
    });
    my $hLangValues = GetSEOCheckValues($Shop, $Art1, $ShopLangID);
    $hValues->{$ShopLangID} = {
        %$hGeneralValues,
        %$hLangValues,
    };

    foreach my $Token ( qw (Url Title Description AdvDescription )) {
        cmp_ok($hValues->{$ShopLangID}->{$Token}->{'Rating'}, '==', 1, 'GetSEOCheckValues Rating 1');
    }
    cmp_ok(CalculateSEOQuality($hValues->{$ShopLangID}, $Art1), '==', 90, 'CalculateSEOQuality returns 90 as image is missing');  ## no critic qw(ProhibitMagicNumbers)

    #
    cmp_ok(GetWordCount('w1 - w2 | word3+word4 _ w5-w6'), '==', 7,  'GetWordCount counts only words');

    # abstracted from a customer's text
    cmp_ok(
        GetWordCount(
         'AB 1234 cccccc ddddd Ffff-Tttttt mmm Bbbb Dddddddd X-Xxx Ffff-Tttttt mmm Bbbb Dddddddd. 1 kkkkkkkkkkkkkk Aaaaaaaa fff Vvvvvvvvvv, eeeee mmm Eeeee Sssssssss. Mmmmmmmm: 100% Ppppppppp Mmmm: cc. 11 cc llll, 11 cc bbbb MM 1234 mmmmmm bbbbb Ffff-Tttttt mmm Bbbb Dddddddd ii 1 Fffffffffffffffff.'
        ),
        '==',
        44,
        'GetWordCount counts only words'
    );

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub Cleanup {
    LoadObjectByPath('/Test')->delete if ExistsObjectByPath('/Test');
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    return 1;
}

RunScript( Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
