use strict;
use Test::More;
use Time::HiRes qw (time);
use XML::SemanticDiff;
use DE_EPAGES::Core::API::Error qw( HandleError );
use DE_EPAGES::Core::API::File  qw( AbsolutePath DeleteFile GetFileContent WriteFile );
use DE_EPAGES::Core::API::Script  qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath JoinObjectPath LoadClassByAlias LoadObjectByPath LoadRootObject ResetFactoryCaches );
use DE_EPAGES::Object::API::XML::ObjectExportDriver qw( ExportObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function     _TimeOK
# state        private
#----------------------------------------------------------------------------------------
# syntax       _TimeOK( $MaxTime, $Comment, $Code );
#----------------------------------------------------------------------------------------
# description  Tests that the code block runs fast enough.
#----------------------------------------------------------------------------------------
# input        $MaxTime | maximum runtime in seconds | float
# input        $Comment | comment | string
# input        $Code | code to execute | code reference
#========================================================================================
sub _TimeOK {
    my ($MaxTime, $Comment, $Code) = @_;

    my $time0 = time();
    $Code->();
    my $Time = time() - $time0;
    cmp_ok( $Time, '<', $MaxTime, sprintf("$Comment: %0.1f s", $Time) );
    return;
}

sub _TestXMLExportImport($$$$) {
    my ($ShopConfigService, $StoreName, $ShopAlias, $NewAlias) = @_;

    # delete the clone shop if it exists from a previous test
    if( $ShopConfigService->exists({ 'Alias' => $NewAlias })->result ) {
        note 'delete previously created clone shop';
        $ShopConfigService->deleteShopRef({ 'Alias' => $NewAlias });
    }

    # Export shop
    my $FileName = 'DemoShop.xml';
    _TimeOK( 11.5, 'export shop', sub {  ## no critic qw(ProhibitMagicNumbers)
        ExportObjects( '/Shops', $FileName, { withdefaults => 1 }, [$ShopAlias], \&_TransForm );
    });

    # Transform old alias into new one.
    my $Content = ${ GetFileContent( $FileName, { utf8 => 1 } ) };
    $Content =~ s/\Q$ShopAlias\E/$NewAlias/g;
    my $ImportFile = 'DemoShopXMLimport.xml';
    WriteFile( $ImportFile, \$Content, { utf8 => 1 } );
    ok( 1, 'transform shop alias' );

    my $NewPath = JoinObjectPath( ['Shops', $NewAlias] );
    if( ExistsObjectByPath( $NewPath ) ) {
        note 'delete shop object of clone shop';
        LoadObjectByPath( $NewPath )->delete;
    }

    # Before creating a new shop, reset the default locale to en_GB
    is( LoadRootObject()->get('LocaleID'), 'en_GB', 'default system locale is en_GB, see DE_EPAGES::Object::API::Cartridge::_activateLocales' );

    my $EmptyFile = 'Empty.xml';
    WriteFile( $EmptyFile, \'<epages />', { utf8 => 1 } );

    # create an empty shop
    _TimeOK( 10, 'create shop', sub {
        $ShopConfigService->create( {
            Database                => $StoreName,
            ShopType                => 'MerchantCorporate',
            ShopAlias               => $NewAlias,
            Alias                   => $NewAlias,
            ImportFiles             => [ AbsolutePath($EmptyFile) ],
        });
    });

    # update cache for store database
    ResetFactoryCaches();

    # Import shop data from XML file
    _TimeOK( 36, 'import xml', sub { ## no critic qw(ProhibitMagicNumbers)
        ImportObjects( $ImportFile, '/Shops' );
    });

    # inform other application servers
    SynchronizeCache();

    # export CSS stylesheets
    my $Shop = LoadObjectByPath( $NewPath );
    _TimeOK( 4.0, 'export CSS stylesheets', sub {
        $_->writeCSS foreach @{ $Shop->get('Styles') };
    });

    # Export shop data again
    my $ExportFile = 'DemoShopXMLexport.xml';
    _TimeOK( 9, 're-export imported shop', sub { ## no critic qw(ProhibitMagicNumbers)
        ExportObjects( '/Shops', $ExportFile, { withdefaults => 1 }, [$NewAlias], \&_TransForm );
    });

    my $Equal = 1;
    my $diff = XML::SemanticDiff->new( keeplinenums => 1, keepdata => 1 );
    foreach my $change ($diff->compare($ImportFile, $ExportFile)) {
        diag "$change->{message} in context $change->{context} (between lines $change->{startline} and $change->{endline})\n";
        diag "  got     : $change->{new_value}\n" if defined $change->{new_value};
        diag "  expected: $change->{old_value}\n" if defined $change->{old_value};
        $Equal = 0;
    }
    ok( $Equal, 'export file equals import file' );

    # cleanup
    SKIP: {
        skip 'don\'t cleanup, because export files are different', 1 unless $Equal;

        _TimeOK( 45, 'deleteShopRef', sub { ## no critic qw(ProhibitMagicNumbers)
            $ShopConfigService->deleteShopRef({ 'Alias' => $NewAlias });
        });

        DeleteFile( $_ ) foreach( $EmptyFile, $FileName, $ImportFile, $ExportFile );
    }
}

RunScript( 'Sub' => sub {

    my $NumTests = 10; ## no critic qw(ProhibitMagicNumbers)
    plan tests => $NumTests;

    my $ShopAlias = 'DemoShop';
    my $StoreName = GetTestSite();
    my $ExistsDemoShop;
    RunOnStore( Store => $StoreName, Sub => sub {
        $ExistsDemoShop = ExistsObjectByPath( JoinObjectPath([ 'Shops', $ShopAlias ]) );

        # set the attribute NewsletterCampaign.TestEMail from Shop.Address.EMail
        # which is done by DE_EPAGES::ShopNewsletter::Hooks::NewsletterCampaign::OnInsertNewsletterCampaign
        # on XML import
        if( $ExistsDemoShop && LoadClassByAlias('Shop')->existsAttribute('NewsletterCampaigns') ) {
            my $Shop = LoadRootObject()->child('Shops')->child($ShopAlias);
            my $Address = $Shop->get('Address');
            if( $Address ) {
                my $Email = $Address->get('EMail');
                if( $Email ) {
                    foreach my $Campaign ( @{$Shop->get('NewsletterCampaigns')} ) {
                        $Campaign->set({ 'TestEMail' => $Email }) unless defined $Campaign->get('TestEMail');
                    }
                }
            }
        }
    });

    SKIP: {
        skip "shop $ShopAlias does not exists in database $StoreName", $NumTests unless $ExistsDemoShop;

        my $User = '/Providers/Distributor/Users/admin';
        my $Password = 'admin';
        my $WebServiceProxy;
        RunOnStore( 'Store' => 'Site', 'Sub' => sub {
            $WebServiceProxy = LoadRootObject()->folder('Stores')->child('Site')->get('WebServiceProxy');
        });
        ok( 1, "get web service proxy ($WebServiceProxy)" );

        # create a web service client
        my $ShopConfigService = DE_EPAGES::WebService::API::Client
            ->uri('urn://epages.de/WebService/ShopConfigService/2009/09')
            ->proxy( $WebServiceProxy, timeout => 900 );
        $ShopConfigService->userinfo("$User:$Password");

        my $NewAlias = 'DemoShopXMLimport';
        RunOnStore( Store => $StoreName, Sub => sub {
            eval { _TestXMLExportImport($ShopConfigService, $StoreName, $ShopAlias, $NewAlias) };
            HandleError( Finally => sub {
                SynchronizeCache();
            });
        });
    }
});


sub _TransForm {
    my ($TagName, $hAttributes) = @_;
    delete $hAttributes->{'GUID'};
    delete $hAttributes->{'CreationDate'};
    return;
}

__END__

rem Prior creating a new shop, reset the default locale to en_GB, remove all import files from the shop type
perl %EPAGES_CARTRIDGES%/DE_EPAGES/Object/Scripts/set.pl -storename Store -path / LocaleID=en_GB
perl %EPAGES_CARTRIDGES%/DE_EPAGES/Object/Scripts/set.pl -storename Site -path /Providers/Distributor/ShopTypes/MerchantCorporate ShopImportFile=

rem Create new and empty shop.
perl %EPAGES_CARTRIDGES%/DE_EPAGES/ShopConfiguration/Scripts/createShop.pl -proxy http://wloch.intern.epages.de/epages/Site.soap -wsuser /Providers/Distributor/Users/admin -passwd admin -alias DemoShopXMLimport -shoptype MerchantCorporate -storename Store

rem Set the default locale of this Store back to de_DE, set the original import files
perl %EPAGES_CARTRIDGES%/DE_EPAGES/Object/Scripts/set.pl -storename Store -path / LocaleID=de_DE
perl %EPAGES_CARTRIDGES%/DE_EPAGES/Object/Scripts/set.pl -storename Site -path /Providers/Distributor/ShopTypes/MerchantCorporate ShopImportFile=Basic.xml,Translation.Basic.de.xml,Translation.Basic.en.xml,StarterContent.xml,Translation.StarterContent.de.xml,Translation.StarterContent.en.xml,Country_DE.xml

rem Import shop.
perl %EPAGES_CARTRIDGES%/DE_EPAGES/Object/Scripts/import.pl -storename Store -path /Shops "c:\temp/DemoShopXMLimport.xml" || echo 'Shop import failed!' >> "c:\temp/shopexportimport.log"

rem We only do the following steps, in case we encountered no errors so far.
rem Compare created shop with imported shop.
rem Export the imported shop first.
rem Create Style first.
perl %EPAGES_CARTRIDGES%/DE_EPAGES/Design/Scripts/saveAllStyles.pl -storename Store

perl %EPAGES_CARTRIDGES%/DE_EPAGES/Object/Scripts/export.pl -withdefaults -withoutguids -withoutcreationdate -storename Store -path /Shops -file "c:\temp/DemoShopXMLimport_exported.xml" DemoShopXMLimport

rem Analysis
winmerge "c:\temp/DemoShopXMLimport.xml" "c:\temp/DemoShopXMLimport_exported.xml"
