#!/bin/env perl
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBPAaXUl5jMabHSgyARAuWPAJ4jn7w60DHb4wrKYQtIm30+Q0GI5ACguit3
#OvMaqn1GX/4/ukZJKmOGax0=
#=Lzf5
#-----END PGP SIGNATURE-----

use strict;
use Config;
use Getopt::Long;
use Sys::Hostname;
use Config::IniFiles;
use File::Temp;
use File::Basename qw( dirname );
use Proc::Background;
use Time::HiRes qw (time);

use DE_EPAGES::Core::API::Process qw( WaitForBackgroundProcesses ShowBackgroundProcesses );
use DE_EPAGES::MailType::API::TextMail qw( SendTextMail );
use DE_EPAGES::Core::API::PerlTools qw( UsePackage );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw( CorrectPathSlashes CreateDirectory DeleteDirectory GetFileContent WriteFile );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );

BEGIN {
    if ( $Config{'osname'} eq 'MSWin32' ) {
        UsePackage( 'Win32::Process::Info' );
    } else {
        UsePackage( 'Proc::ProcessTable' );
    }
}

use constant ATTACH_BIG_FILE => 50000; # compess and attach files greater than 50 kB
my $START_TIME = time(); # time when scheduler is started

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [Options] SECTION

Options:
    -help    show the command line options
    -ini     read configuration file (default: $ENV{EPAGES_CONFIG}/Scheduler.conf)
    -var     directory for variable data (default: VarDir in -ini)
    -maxage AGE  max job duration without a next instance reports an error
                 AGE is one of (default: MaxDuration in -ini):
                 <MINUTES> (e.g. -d 56)
                 <HOURS>:<MINUTES> (e.g. -d 7:30)
                 <DAYS>-<HOURS>:<MINUTES> (e.g. -d 3-00:00)
    SECTION  section in configuration file

Example:
    perl $0
END_USAGE
;
    exit 2;
}

sub Main {
    local $| = 1;

    my $Options = _OptionsFromCommandLine([qw(ini=s var=s maxage=s help)]);

    my %Ini;
    tie %Ini, 'Config::IniFiles', ( -file => $Options->{ini}, -default => ':Default:' )
        or die "$Options->{ini}: $!";

    _OptionsFromIniFile($Options, \%Ini);

    exit 0 if $Options->{isactive} == 0;

    CreateDirectory($Options->{var}) unless -d $Options->{var};

    exit 0 if _RunningProcessFound($Options, \%Ini);

    exit _RunPerlScript($Options, \%Ini);
}

#========================================================================================
# function     _OptionsFromCommandLine
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Options = _OptionsFromCommandLine($GetOpt);
# example      $Options = _OptionsFromCommandLine([qw(ini=s var=s maxage=s help)]);
#----------------------------------------------------------------------------------------
# description  The function sets key/value pairs in $Options derived from command line.
#----------------------------------------------------------------------------------------
# input        $GetOpt | option descriptions as used by GetOptions | ref.array
# return       $Options | key/value pairs
#                   <ul>
#                       <li>ini       - file name of Scheduler.conf
#                       <li>var       - directory for variable data
#                       <li>maxage    - max duration without a next instance reports an error
#                       <li>section   - section name in Scheduler.conf
#                       <li>hostname  - short version of FQDN
#                   </ul>
#               | ref.hash
#========================================================================================
sub _OptionsFromCommandLine ($) {
    my ($GetOpt) = @_;
    my $Options = {};

    GetOptions($Options, @$GetOpt) or usage();
    usage() if $Options->{help};
    usage() unless scalar @ARGV == 1;

    $Options->{section} = $ARGV[0];
    $Options->{ini} = "$ENV{EPAGES_CONFIG}/Scheduler.conf" unless $Options->{ini};
    usage() unless -r $Options->{ini};

    $Options->{hostname} = hostname();
    chomp($Options->{hostname});
    $Options->{hostname} =~ s/\..*//;

    return $Options;
}

#========================================================================================
# function     _OptionsFromIniFile
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Options = _OptionsFromIniFile($Options, $Ini);
# example      _OptionsFromIniFile($Options, \%Ini);
#----------------------------------------------------------------------------------------
# description  The function sets key/value pairs in $Options derived from Scheduler.conf hash
#----------------------------------------------------------------------------------------
# input        $Options | hash as defined by _OptionsFromCommandLine | ref.hash
# input        $Ini     | Scheduler.conf hash | ref.hash
# return       $Options | key/value pairs
#                   <ul>
#                       <li>ini       - file name of Scheduler.conf
#                       <li>var       - directory for variable data
#                       <li>maxage    - max duration without a next instance reports an error
#                       <li>section   - section name in Scheduler.conf
#                       <li>hostname  - short version of FQDN
#                       <li>isactive  - if set to 0, do not run scheduler
#                       <li>varfile   - name of variable data files (prefix), suffixed by:
#                                       -- .log - contains log messages of all scheduler instances
#                                       -- .pid - contains pid of running scheduler
#                       <li>tmpfile   - name of temporary data files (prefix), suffixed by:
#                                       -- .run - contains log messages of running scheduler
#                                       -- .mail - contains error mail of running scheduler
#                   </ul>
#               | ref.hash
#========================================================================================
sub _OptionsFromIniFile ($$) {
    my ($Options, $Ini) = @_;
    my $Section = $Options->{section};

    $Options->{maxage} = $Ini->{$Section}{MaxDuration} unless $Options->{maxage};

    $Options->{isactive} = $Ini->{$Section}{IsActive};
    if ($Options->{isactive} != 0) {
        $Options->{isactive} = $Ini->{'LOOPS'}{IsActive} if defined $Ini->{'LOOPS'}{IsActive};
        $Options->{isactive} = $Ini->{':Default:'}{IsActive} if defined $Ini->{':Default:'}{IsActive};
    }

    my $LocalVarDir = "VarDir:$Options->{hostname}";
    $Options->{var} = $Ini->{$Section}{$LocalVarDir} unless defined $Options->{var};
    $Options->{var} = $Ini->{$Section}{VarDir} unless defined $Options->{var};
    $Options->{var} = $ENV{EPAGES_LOG} unless $Options->{var};
    $Options->{varfile} = "$Options->{var}/$Options->{hostname}.$Options->{section}";

    my $TempDir = File::Temp->newdir();
    $Options->{tmpfile} = CorrectPathSlashes("$TempDir/$Options->{hostname}.$Options->{section}");

    return $Options;
}

#========================================================================================
# function     _ReportError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Result = _ReportError($ErrorMsg, $Options, $Ini, $RunFile);
# example      return _ReportError($ErrorMsg, $Options, $Ini);
#----------------------------------------------------------------------------------------
# description  Write/send $ErrorMsg to $LogFile/e-mail
#----------------------------------------------------------------------------------------
# input        $ErrorMsg | error message | string
# input        $Options  | key/value pairs as defined by _OptionsFrom{CommandLine,IniFile} | ref.hash
# input        $Ini      | Scheduler.conf hash | ref.hash
# input        $RunFile  | if defined, add contents to $LogFile/e-mail | string
# return       $Result   | always 1 | integer
#========================================================================================
sub _ReportError {
    my ($ErrorMsg, $Options, $Ini, $RunFile) = @_;

    my $LogFile = "$Options->{varfile}.log";
    my $MailFile = "$Options->{tmpfile}.mail";

    # check that $MailFile is writeable
    WriteFile($MailFile, \1, {create_directory=>1});

    # header
    my $WriteLogFile = 1;
    my $Header = _ReportHeader($Options, $Ini);
    WriteFile($MailFile, \$Header);
    eval { WriteFile($LogFile, \$Header, {create_directory=>1}); };
    if (ExistsError()) {
        warn GetError()->message;
        $WriteLogFile = 0;
    }

    # add error message to body
    WriteFile($MailFile, \$ErrorMsg, {append => 1});
    WriteFile($LogFile, \$ErrorMsg, {append => 1}) if $WriteLogFile;

    my $SendFile = $MailFile;
    my ($Descr, $RunFileSize);
    $RunFileSize = (stat($RunFile))[7] if -s $RunFile;

    # add run file to body
    if ($RunFileSize) {
        my $rContent = GetFileContent($RunFile);
        WriteFile($LogFile, $rContent, {append => 1}) if $WriteLogFile;
        if ($RunFileSize > ATTACH_BIG_FILE) {
            $Descr = "$Header: /CVS/RND/epages/epages5/AllOS/Scheduler/bin/wrapScheduler.PL,v 1.13.2.5 2011/12/15 14:46:46 wloch Exp $ErrorMsg";
            $SendFile = $RunFile;
        } else {
            WriteFile($MailFile, $rContent, {append => 1});
        }
    }

    # footer
    my $Footer =  "\nENDDATE:   ".localtime(time)."\n\n";
    WriteFile($LogFile, \$Footer, {append => 1}) if $WriteLogFile;
    $Descr .= $Footer if $Descr;

    # send error mail
    _SendErrorMail($Options, $Ini, $SendFile, $Descr);
    return 1;
}

#========================================================================================
# function     _SendErrorMail
# state        private
#----------------------------------------------------------------------------------------
# syntax       _SendErrorMail($Options, $Ini, $SendFile, $Descr);
#----------------------------------------------------------------------------------------
# description  Sends $SendFile depending on $Options, $Ini and $Descr.
#               If $Descr is empty, $SendFile is sent as mail message.
#               Else, mail message is $Descr and $SendFile is compressed and attached.
#----------------------------------------------------------------------------------------
# input        $Options  | key/value pairs as defined by _OptionsFrom{CommandLine,IniFile} | ref.hash
# input        $Ini      | Scheduler.conf hash | ref.hash
# input        $SendFile | file to send
# input        $Descr    | description for attached $SendFile
#========================================================================================
sub _SendErrorMail {
    my ($Options, $Ini, $SendFile, $Descr) = @_;
    my $Section = $Options->{section};

    my $Subject = "$Options->{hostname}: Output from program $Options->{section}";
    my %Args = ( ini => $Options->{ini}, section => $Section, subject => $Subject, );

    if (defined $Descr) {
        $Args{attachment} = 1;
        system("gzip -9 $SendFile");
        $Args{file} = "$SendFile.gz";
        $Args{descr} = $Descr;
    } else {
        $Args{file} = $SendFile;
    }

    my $Host = $Options->{hostname};
    foreach my $Key (qw(from to store)) {
        my $SectionKey = 'MailStore';
        $SectionKey = 'MailSender' if $Key eq 'from';
        $SectionKey = 'MailRecipient' if $Key eq 'to';
        $Args{$Key} = $Ini->{$Section}{"$SectionKey:$Host"} and next;
        $Args{$Key} = $Ini->{$Section}{$SectionKey};
    }

    my $ArgString;
    foreach  my $Key (qw(subject from to store file attachment)) {
        $ArgString .= " $Key='$Args{$Key}'" if $Args{$Key};
    }
    SendTextMail( \%Args );
}

#========================================================================================
# function     _ReportHeader
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Header = _ReportHeader($Options, $Ini);
#----------------------------------------------------------------------------------------
# description  Returns string that is put on the beginning of a scheduler report. The
#               string contains information derived from $Options and $Ini.
#----------------------------------------------------------------------------------------
# input        $Options  | key/value pairs as defined by _OptionsFrom{CommandLine,IniFile} | ref.hash
# input        $Ini      | Scheduler.conf hash | ref.hash
# return       $Header   | header string | string
#========================================================================================
sub _ReportHeader {
    my ($Options, $Ini) = @_;
    my $Section = $Options->{section};

    my $Date = scalar localtime($START_TIME);
    my $RunFile = "$Options->{tmpfile}.run";
    my ($PerlExe, $UserName) = _ProcessInfo();
    my $Script = $Ini->{$Section}{Command};
    my $Args = $Ini->{$Section}{Options};

    my $LoopString;
    if ($Ini->{$Section}{Loop}) {
        my $Loop = $Ini->{$Section}{Loop};
        $Loop = join("\n  ", split(/,/, $Ini->{LOOPS}{$Loop}));
        $LoopString =  "\nLOOP: $Ini->{$Section}{Loop}=$Loop" if ($Loop =~ /\S/);
    }

    return <<EOF
*** Output from program $Section ***
COMMAND: $PerlExe $Script $Args > $RunFile 2>&1$LoopString
HOSTNAME: $Options->{hostname}
USERNAME: $UserName
DATE: $Date

EOF
;
}

#========================================================================================
# function     _ProcessInfo
# state        private
#----------------------------------------------------------------------------------------
# syntax       my ($PerlExe, $UserName) = _ProcessInfo();
#----------------------------------------------------------------------------------------
# description  Returns perl executable and user that runs the process.
#----------------------------------------------------------------------------------------
# return       $PerlExe  | path to perl executable | string
# return       $UserName | namek of the user that runs the process | string
#========================================================================================
sub _ProcessInfo {
    return ($ENV{PERL}, (getpwuid($>))[0]) if $Config{osname} ne 'MSWin32';

    my $pi = Win32::Process::Info->new();
    foreach my $Proc (@{$pi->GetProcInfo()}) {
        next unless $Proc->{ProcessId} == $$;
        return ($^X, $Proc->{Owner});
    }
    return ($^X, undef);
}

#========================================================================================
# function     _RunningProcessFound
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Result = _RunningProcessFound($Options, $Ini);
# example      exit 0 if _RunningProcessFound($Options, \%Ini);
#----------------------------------------------------------------------------------------
# description  Terminate scheduler if the previous scheduler instance is still running
#----------------------------------------------------------------------------------------
# input        $Options | key/value pairs as defined by _OptionsFrom{CommandLine,IniFile} | ref.hash
# input        $Ini     | Scheduler.conf hash | ref.hash
# return       $Result  | return value; 0 -> success, else -> failure | integer
#========================================================================================
sub _RunningProcessFound {
    my ($Options, $Ini) = @_;

    my $PidFile = "$Options->{varfile}.pid";
    return 0 unless -e $PidFile;

    my $Pid = eval { ${GetFileContent($PidFile)} };
    my $ErrorMsg = ExistsError() ? GetError()->message : undef;
    return _ReportError($ErrorMsg, $Options, $Ini) if $ErrorMsg;

    $Pid =~ s/[^0-9]//sg;
    return 0 unless $Pid;

    my $MaxAgeInSeconds = 0;
    if ($Options->{maxage}) {
        my ($Minutes, $Hours, $Days);
        $Minutes = $Options->{maxage};
        $Hours = $1 if $Minutes =~ s/(.*)://; ## no critic qw(ProhibitCaptureWithoutTest)
        $Days = $1 if $Hours =~ s/(.*)-//; ## no critic qw(ProhibitCaptureWithoutTest)

        $MaxAgeInSeconds += $Minutes if $Minutes;
        $MaxAgeInSeconds += $Hours * 60 if $Hours; ## no critic qw(ProhibitMagicNumbers)
        $MaxAgeInSeconds += $Days * 24 * 60;       ## no critic qw(ProhibitMagicNumbers)
        $MaxAgeInSeconds *= 60;                    ## no critic qw(ProhibitMagicNumbers)
    }

    $ErrorMsg = _FindRunningWindowsProcess($Pid, $PidFile, $MaxAgeInSeconds) if $Config{osname} eq 'MSWin32';
    $ErrorMsg = _FindRunningLinuxProcess($Pid, $PidFile, $MaxAgeInSeconds) if $Config{osname} ne 'MSWin32';

    return 0 unless defined $ErrorMsg;
    return 1 unless $ErrorMsg;

    $ErrorMsg = <<EOF
Outdated scheduler ID file found.
Check the corresponding processes and stop them if necessary.
Delete the scheduler ID file, if the processes have stopped.

$ErrorMsg

EOF
;
    return _ReportError($ErrorMsg, $Options, $Ini);
}

#========================================================================================
# function     _FindRunningWindowsProcess
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ErrorMsg = _FindRunningWindowsProcess($Pid, $PidFile, $MaxAgeInSeconds);
#----------------------------------------------------------------------------------------
# description  Searches $Pid in Win32::Process::Info(). Returns undef unless found.
#               Returns empty string if found but process is younger than $MaxAgeInSeconds.
#               Returns error message if found and process is older than $MaxAgeInSeconds.
#----------------------------------------------------------------------------------------
# input        $Pid             | process id for the process to search | integer
# input        $PidFile         | name of file that contains pid | string
# input        $MaxAgeInSeconds | timeout for process | integer
# return       $ErrorMsg        | undef   -> $Pid does not exist
#                                  empty   -> $Pid exists and process is young
#                                  message -> $Pid exists and process is old   | string
#========================================================================================
sub _FindRunningWindowsProcess {
    my ($Pid, $PidFile, $MaxAgeInSeconds) = @_;

    my $LastProc;
    my $pi = Win32::Process::Info->new();
    foreach my $Proc (@{$pi->GetProcInfo()}) {
        if ($Proc->{ProcessId} == $Pid) {
            return '' unless $MaxAgeInSeconds;
            return '' if $START_TIME - $Proc->{CreationDate} < $MaxAgeInSeconds;
            $LastProc = $Proc;
            last;
        }
    }
    return undef unless $LastProc;

    my $CreationDate = localtime($LastProc->{CreationDate});
    my $CurrentDate = localtime($START_TIME);
    return <<EOF
PidFile      : $PidFile
CSName       : $LastProc->{CSName}
CommandLine  : $LastProc->{CommandLine}
Owner        : $LastProc->{Owner}
CreationDate : $CreationDate
CurrentDate  : $CurrentDate
MaxDuration  : $MaxAgeInSeconds seconds
ProcessId    : $Pid
EOF
;
}

#========================================================================================
# function     _FindRunningLinuxProcess
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ErrorMsg = _FindRunningLinuxProcess($Pid, $MaxAgeInSeconds);
#----------------------------------------------------------------------------------------
# description  Searches $Pid in Proc::ProcessTable(). Returns undef unless found.
#               Returns empty string if found but process is younger than $MaxAgeInSeconds.
#               Returns error message if found and process is older than $MaxAgeInSeconds.
#----------------------------------------------------------------------------------------
# input        $Pid             | process id for the process to search | integer
# input        $PidFile         | name of file that contains pid | string
# input        $MaxAgeInSeconds | timeout for process | integer
# return       $ErrorMsg        | undef   -> $Pid does not exist
#                                  empty   -> $Pid exists and process is young
#                                  message -> $Pid exists and process is old   | string
#========================================================================================
sub _FindRunningLinuxProcess {
    my ($Pid, $PidFile, $MaxAgeInSeconds) = @_;

    my $LastProc;
    my $pi = Proc::ProcessTable->new;
    foreach my $Proc (@{$pi->table}) {
        if ($Proc->pid == $Pid) {
            return '' unless $MaxAgeInSeconds;
            return '' if $START_TIME - $Proc->start < $MaxAgeInSeconds;
            $LastProc = $Proc;
            last;
        }
    }
    return undef unless $LastProc;

    my $CreationDate = localtime($LastProc->start);
    my $CurrentDate = localtime($START_TIME);
    my $Owner = (getpwuid($LastProc->uid))[0];
    my $Group = (getgrgid($LastProc->gid))[0];
    my $CommandLine = $LastProc->cmndline;
    my $State = $LastProc->state;
    my $WorkingDir = $LastProc->cwd;
    return <<EOF
PidFile      : $PidFile
CommandLine  : $CommandLine
State        : $State
WorkingDir   : $WorkingDir
Owner        : $Owner
Group        : $Group
CreationDate : $CreationDate
CurrentDate  : $CurrentDate
MaxDuration  : $MaxAgeInSeconds seconds
ProcessId    : $Pid
EOF
;
}

#========================================================================================
# function     _ReplaceENV
# state        private
#----------------------------------------------------------------------------------------
# syntax       $OutString = _ReplaceENV( $InString );
#----------------------------------------------------------------------------------------
# description  Replaces '$ENV{Varname}' in $InString by the actual values of environment
#               variables and returns $OutString
#----------------------------------------------------------------------------------------
# input        $InString        | string $ENV{Varname} | string
# return       $OutString       | $InString with replaced $ENV{Varname} | string
#========================================================================================
sub _ReplaceENV {
    my ($String) = @_;
    $String =~ s/\$ENV{(.*?)}/$ENV{$1}/ge;
    return $String;
}

#========================================================================================
# function     _RunPerlScript
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Result = _RunPerlScript($Options, $Ini);
# example      exit _RunPerlScript($Options, \%Ini);
#----------------------------------------------------------------------------------------
# description  Runs perl scripts as defined in $Options and $Ini.
#----------------------------------------------------------------------------------------
# input        $Options | key/value pairs as defined by _OptionsFrom{CommandLine,IniFile} | ref.hash
# input        $Ini     | Scheduler.conf hash | ref.hash
# return       $Result  | return value; 0 -> success, else -> failure | integer
#========================================================================================
sub _RunPerlScript {
    my ($Options, $Ini) = @_;
    my $Section = $Options->{section};

    my $PidFile = "$Options->{varfile}.pid";
    my $RunFile = "$Options->{tmpfile}.run";

    # write .pid/.run file
    foreach my $File ($PidFile, $RunFile) {
        my $Content = $File eq $PidFile ? $$ : '';
        eval { WriteFile($File, \$Content, {create_directory=>1}); };
        if( ExistsError() ) {
            return _ReportError(GetError()->message, $Options, $Ini);
        }
    }

    my ($PerlExe, undef) = _ProcessInfo();
    my $PerlCommand = _GetPerlCommand($Options, $Ini);

    # run $PerlExe
    my ($LoopCount, $Running) = (0, 0);
    my $ProcessList = [];
    my $AllProcesses = [];
    my $Loop = $Ini->{$Section}{Loop};
    if ($Loop) {
        foreach my $LoopArgs (split(/,/, $Ini->{LOOPS}{$Loop})) {
            $LoopCount++;
            $LoopArgs = _ReplaceENV( $LoopArgs );
            # can't use operator ">>" for parallel processes on Windows
            my $Command = sprintf '%s %s %s > "%s-%06d" 2>&1'
                , $PerlExe, $PerlCommand, $LoopArgs, $RunFile, $LoopCount;
            if( $Config{'osname'} eq 'MSWin32' ) {
                # can't use operators ">" in Proc::Background on Windows
                $Command = "$ENV{'ComSpec'} /C $Command";
            }
            my $Proc = Proc::Background->new($Command);
            printf "%6d: %s\n", $Proc->pid, $Command if $Ini->{$Section}{Debug};

            push(@$ProcessList, $Proc);
            push(@$AllProcesses, $Proc);
            ShowBackgroundProcesses($AllProcesses, $START_TIME) if $Ini->{$Section}{Debug};
            $ProcessList = WaitForBackgroundProcesses($ProcessList,
                                                      { timeout => $Ini->{$Section}{JobDuration},
                                                        parallel => $Ini->{$Section}{ParallelJobs},
                                                      });
        }
    }
    if ($LoopCount) {
        $Running = ShowBackgroundProcesses($AllProcesses, $START_TIME) if $Ini->{$Section}{Debug};
        WaitForBackgroundProcesses($ProcessList, { timeout => $Ini->{$Section}{JobDuration} })
            if $Ini->{$Section}{ParallelJobs};
        ShowBackgroundProcesses($AllProcesses, $START_TIME) if $Ini->{$Section}{Debug} && $Running;
        WriteFile($RunFile, GetFileContent($_), { append => 1}) foreach glob("$RunFile-*");
    } else {
        SystemCommand("$PerlExe $PerlCommand > \"$RunFile\" 2>&1", {'Print' => $Ini->{$Section}{Debug}});
    }

    _ReportError(undef, $Options, $Ini, $RunFile) if -s $RunFile;

    # delete .pid file
    unlink $PidFile;
    return 1 if -s $RunFile;

    # delete temp dir
    DeleteDirectory(dirname($Options->{tmpfile}), {'recursive' => 1});
    return 0;
}

#========================================================================================
# function     _GetPerlCommand
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $PerlCommand = _GetPerlCommand($Options, $Ini);
#----------------------------------------------------------------------------------------
# description  Returns perl script with arguments as defined in $Options and $Ini.
#----------------------------------------------------------------------------------------
# input        $Options | key/value pairs as defined by _OptionsFrom{CommandLine,IniFile} | ref.hash
# input        $Ini     | Scheduler.conf hash | ref.hash
# return       $PerlCommand  |  perl script with arguments | string
#========================================================================================
sub _GetPerlCommand {
    my ($Options, $Ini) = @_;
    my $Section = $Options->{section};
    my $Host = $Options->{hostname};

    my $Args = _ReplaceENV( $Ini->{$Section}{Options} );
    $Args =~ s,-nooutput,, if $Ini->{$Section}{Debug};

    return _ReplaceENV( $Ini->{$Section}{"Command:$Host"} ). " $Args"
        if $Ini->{$Section}{"Command:$Host"};
    return _ReplaceENV( $Ini->{$Section}{Command} ). " $Args";
}

RunScript( 'Sub' => \&Main );

1;
